@@ -25,7 +25,13 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/param.h>
+#include <sys/mount.h>
+#endif
+#ifdef HAVE_SYS_VFS_H
 #include <sys/vfs.h>
+#endif
 
 #include <unistd.h>
 #include <inttypes.h>
@@ -161,8 +167,8 @@ public:
 	tDlDescRemote(cmstring &p, UINT n) : tDlDesc(p,n), bIsFirst(true)
 	{
 		// expire the caches every time, should not cost much anyway
-		tcpconnect::ExpireCache();
-		CAddrInfo::ExpireCache();
+		tcpconnect::BackgroundCleanup();
+		CAddrInfo::BackgroundCleanup();
 	};
 
 	int Read(char *retbuf, const char *path, off_t pos, size_t len)
@@ -172,7 +178,7 @@ public:
 		uri.sPath += baseUrl.sHost
 		// + ":" + ( baseUrl.sPort.empty() ? baseUrl.sPort : "80")
 				+ baseUrl.sPath + m_path;
-		class tFitem: public fileitem_base
+		class tFitem: public fileitem
 		{
 		public:
 			char *pRet;
@@ -218,7 +224,7 @@ public:
 			}
 #define SETERROR { nErr=__LINE__; return false;}
 			bool &m_isFirst;
-			SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &head, const char*, bool bRestarted)
+			bool DownloadStartedStoreHeader(const header &head, const char*, bool bRestarted, bool&)
 			{
 				_cerr(head.frontLine<<endl);
 				m_head = head; // XXX: bloat, only status line and contlen required
@@ -298,7 +304,7 @@ public:
 		tFileId fidOrig=fid;
 
 		tFitem *pFi = new tFitem(retbuf, len, pos, fid, bIsFirst);
-		tFileItemPtr spFi(static_cast<fileitem_base*>(pFi));
+		tFileItemPtr spFi(static_cast<fileitem*>(pFi));
 		dler.AddJob(spFi, uri);
 		dler.WorkLoop();
 		int nHttpCode(100);
@@ -345,7 +351,7 @@ public:
 		uri.sPath += baseUrl.sHost
 		// + ":" + ( baseUrl.sPort.empty() ? baseUrl.sPort : "80")
 				+ baseUrl.sPath + m_path;
-		class tFitemProbe: public fileitem_base
+		class tFitemProbe: public fileitem
 		{
 		public:
 			ssize_t SendData(int, int, off_t&, size_t)
@@ -360,7 +366,7 @@ public:
 			{
 				m_bHeadOnly = true;
 			}
-			SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &head, const char*, bool bRestart)
+			bool DownloadStartedStoreHeader(const header &head, const char*, bool bRestart, bool&)
 			{
 				if(bRestart)
 					return true;
@@ -370,16 +376,16 @@ public:
 				return true;
 			}
 		};
-		tFileItemPtr probe(static_cast<fileitem_base*>(new tFitemProbe()));
+		tFileItemPtr probe(static_cast<fileitem*>(new tFitemProbe()));
 		dler.AddJob(probe, uri);
 		dler.WorkLoop();
 		int nHttpCode(100);
-		FiStatus res = probe->WaitForFinish(&nHttpCode);
+		fileitem::FiStatus res = probe->WaitForFinish(&nHttpCode);
 		stbuf.st_size = atoofft(probe->GetHeaderUnlocked().h[header::CONTENT_LENGTH], 0);
 		stbuf.st_mode &= ~S_IFDIR;
 		stbuf.st_mode |= S_IFREG;
 
-		if (res < FIST_COMPLETE)
+		if (res < fileitem::FIST_COMPLETE)
 			return -EIO;
 		else if (nHttpCode == 200)
 		{
@@ -924,7 +930,7 @@ int main(int argc, char *argv[])
    return my_fuse_main(argc, argv);
 }
 
-
+#ifndef DEBUG
 // for the uber-clever GNU linker and should be removed by strip again
 namespace aclog
 {
@@ -944,3 +950,4 @@ void err(const char *s, const char* z)
 
 };
 }
+#endif
