@@ -44,7 +44,7 @@ class acbuf
 
         //! like move but also feels unused memory with zeroes
         void flush() { move(); memset(m_buf+w, 0, m_nCapacity-w); }
-                
+
         /*!
          * Writes to a (non-blocking) file descriptor, cares about EAGAIN and updates
          * position indexes. Optional chunklen variable specifies
@@ -76,8 +76,8 @@ class acbuf
 
 /* This is a light-weight and less cumbersome implementation of ostringstream.
  *
- * What it also makes possible... using as string, using alternative add() operators
- * for strings which take optional length, to make the thing work faster with C-strings.
+ * What it also makes possible: use itself as a string, use alternative add() operators
+ * for strings which can also specify the length, and it runs faster with zero-terminated strings.
  */
 class tSS : public acbuf
 {
@@ -95,6 +95,9 @@ public:
 	inline tSS & operator<<(unsigned long val) __tss_nbrfmt("%lu", "%lx", val);
 	inline tSS & operator<<(long long val) __tss_nbrfmt("%lld", "%llx", val);
 	inline tSS & operator<<(unsigned long long val) __tss_nbrfmt("%llu", "%llx", val);
+#ifdef DEBUG
+	inline tSS & operator<<(void* val) __tss_nbrfmt("ptr:%llu", "ptr:0x%llx", (long long unsigned) val);
+#endif
 
     enum fmtflags {	hex, dec };
     inline tSS & operator<<(fmtflags mode) { m_fmtmode=mode; return *this;}
