@@ -32,7 +32,7 @@ tmpDontcacherq, tmpDontcachetgt, tmpDontcache, mirrorsrcs, requestapx;
 extern int debug, numcores, offlinemode, foreground, verbose, stupidfs, forcemanaged, keepnver,
 verboselog, extreshhold, exfailabort, tpstandbymax, tpthreadmax, dnscachetime, dlbufsize, usewrap,
 exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap, dirperms, fileperms,
-maxtempdelay, redirmax, vrangeops, stucksecs;
+maxtempdelay, redirmax, vrangeops, stucksecs, persistoutgoing, pipelinelen, exsupcount;
 
 // processed config settings
 extern tHttpUrl proxy_info;
@@ -72,10 +72,10 @@ struct tRepoData
 
 const tRepoData * GetBackendVec(cmstring &vname);
 
-time_t ExecutePostponed();
+time_t BackgroundCleanup();
 
 extern tStrMap localdirs;
-extern NoCaseStringMap mimemap;
+cmstring & GetMimeType(cmstring &path);
 #define TCP_PORT_MAX 65536
 extern MYSTD::bitset<TCP_PORT_MAX> *pUserPorts;
 
@@ -99,5 +99,9 @@ bool MatchUncacheableTarget(const mstring &);
 bool CompileExpressions();
 }
 
+#define CACHE_BASE (acfg::cacheDirSlash)
+#define CACHE_BASE_LEN (CACHE_BASE.length()) // where the relative paths begin
+#define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
+#define SABSPATH(x) (CACHE_BASE+(x))
 
 #endif
