@@ -28,10 +28,10 @@ private:
 	t_logger(const t_logger&);
 	t_logger operator=(const t_logger&);
 };
-#define USRDBG(nix, msg) LOG(msg)
+#define USRDBG(msg) LOG(msg)
 #else
 // print some extra things when user wants debug with non-debug build
-#define USRDBG(nix, msg) { if(acfg::debug & LOG_DEBUG) {aclog::err( tSS()<<msg); } }
+#define USRDBG(msg) { if(acfg::debug & LOG_DEBUG) {aclog::err( tSS()<<msg); } }
 #endif
 
 
