@@ -18,11 +18,6 @@ class dlcon;
 class tDlJobHints;
 struct foo;
 
-#define CACHE_BASE (acfg::cacheDirSlash)
-#define CACHE_BASE_LEN (CACHE_BASE.length()) // where the relative paths begin
-#define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
-#define SABSPATH(x) (CACHE_BASE+(x))
-
 #define FAKEDATEMARK "Sat, 26 Apr 1986 01:23:39 GMT+3"
 
 // caching all relevant file identity data and helper flags in such entries
@@ -99,7 +94,7 @@ public:
 	class tGetItemDelegate
 	{
 		public:
-			virtual operator tFileItemPtr()=0;
+			virtual fileItemMgmt GetFileItemPtr()=0;
 	};
 
 protected:
@@ -120,13 +115,13 @@ protected:
 	struct tIfileAttribs
 	{
 		bool vfile_ondisk:1, uptodate:1, parseignore:1, hideDlErrors:1, forgiveDlErrors:1,
-		alreadyparsed:1, misc:1;
+		alreadyparsed:1;
 		enumIndexType eIdxType:8;
 		const tStrDeq *bros;
 		off_t space;
 		inline tIfileAttribs() : vfile_ondisk(false), uptodate(false), parseignore(false),
 				hideDlErrors(false), forgiveDlErrors(false), alreadyparsed(false),
-				misc(false), eIdxType(EIDX_UNSUPPORTED), bros(NULL), space(0)
+				eIdxType(EIDX_UNSUPPORTED), bros(NULL), space(0)
 		{};
 	};
 
@@ -141,7 +136,7 @@ protected:
 	void UpdateIndexFiles();
 	void _BusyDisplayLogs();
 	void _Usermsg(mstring m);
-	void AddIFileCandidate(const mstring &sFileRel);
+	bool AddIFileCandidate(const mstring &sFileRel);
 
 	// NOOP, implemented here for convinience
 	bool ProcessOthers(const mstring &sPath, const struct stat &);
@@ -186,7 +181,7 @@ protected:
 	bool ParseAndProcessIndexFile(ifileprocessor &output_receiver,
 			const mstring &sPath, enumIndexType idxType);
 
-	MYMAP<mstring,bool> m_forcedTrashMap;
+	MYMAP<mstring,bool> m_forceKeepInTrash;
 
 	bool GetAndCheckHead(cmstring & sHeadfile, cmstring &sFilePathRel, off_t nWantedSize);
 	bool Inject(cmstring &fromRel, cmstring &toRel,
@@ -200,7 +195,7 @@ protected:
 	{
 		if (++m_nProgIdx == m_nProgTell)
 		{
-			SendFmt()<<"Scanning, found "<<m_nProgIdx<<" file"
+			SendFmt<<"Scanning, found "<<m_nProgIdx<<" file"
 					<< (m_nProgIdx>1?"s":"") << "...<br />\n";
 			m_nProgTell*=2;
 		}
