@@ -27,7 +27,7 @@ protected:
 	set<tFileNdir> m_trashCandHeadSet; // just collect the list of seen head files
 
 	void RemoveAndStoreStatus(bool bPurgeNow);
-	void LoadTrashMapFromFile(bool bForceInsert);
+	void LoadPreviousData(bool bForceInsert);
 
 	// callback implementations
 	virtual void Action(const mstring &);
@@ -40,7 +40,7 @@ protected:
 	virtual void UpdateFingerprint(const mstring &sPathRel, off_t nOverrideSize,
 				uint8_t *pOverrideSha1, uint8_t *pOverrideMd5);
 
-	void LoadIgnoreList();
+	void LoadHints();
 
 	void PurgeMaintLogs();
 
@@ -48,6 +48,10 @@ protected:
 
 	std::ofstream m_damageList;
 	bool m_bIncompleteIsDamaged;
+
+private:
+	int m_nPrevFailCount;
+	bool CheckAndReportError();
 };
 
 #endif /*EXPIRATION_H_*/
