@@ -43,7 +43,7 @@ typedef unsigned char UCHAR;
 #define SZPATHSEPUNIX "/"
 #define CPATHSEPWIN '\\'
 #define SZPATHSEPWIN "\\"
-extern mstring sPathSep;
+extern cmstring sPathSep, sPathSepUnix;
 
 #ifdef WINDOWS
 #define WIN32
@@ -161,12 +161,12 @@ bool ParseKeyValLine(const mstring & sIn, mstring & sOutKey, mstring & sOutVal);
 class tHttpUrl
 {
 public:
-	bool SetHttpUrl(mstring uri);
+	bool SetHttpUrl(cmstring &uri);
 	/*
 	/// The specified url has a path which is a prefix of the one in *this 
 	UINT HasPathPrefix(const tHttpUrl &another);
 	*/
-	mstring ToURI() const;
+	mstring ToURI(bool bEscaped) const;
 	mstring sHost, sPort, sPath;
 	tHttpUrl & operator=(const tHttpUrl &a) 
 	{
@@ -183,9 +183,9 @@ public:
 	}
 	inline void clear() { sHost=sPort=sPath=""; }
 	
-	tHttpUrl(const mstring & host, const mstring & path)
+	/*tHttpUrl(const mstring & host, const mstring & path)
 	: sHost(host), sPath(path)
-	{}
+	{}*/
 	tHttpUrl()
 	{};
 	
@@ -205,15 +205,15 @@ typedef const unsigned char CUCHAR;
 bool CsEqual(const char *a, uint8_t b[], unsigned short binLength);
 void StrSubst(mstring &contents, const mstring &from, const mstring &to);
 
-typedef unsigned int FiStatus;
-
 #if SIZEOF_LONG == 8
-// _FILE_OFFSET_BITS apparently irrelevant, at least on Linux. But if set, watch out for user's experiments.
+// _FILE_OFFSET_BITS mostly irrelevant. But if it's set, watch out for user's "experiments".
 #if _FILE_OFFSET_BITS == 32
 #error Unsupported: _FILE_OFFSET_BITS == 32 with large long size
+#else
+#define OFF_T_FMT "%"PRId64
 #endif
 
-#else // not 64bit arch?
+#else // not a 64bit arch?
 
 #if 64 == _FILE_OFFSET_BITS
 #define OFF_T_FMT "%"PRId64
@@ -251,10 +251,17 @@ tStrDeq ExpandFilePattern(const mstring &pattern, bool bSorted=false);
 //void MakeAbsolutePath(mstring &dirToFix, const mstring &reldir);
 
 
-void HttpEscape(mstring &s);
-bool UrlDecode(const mstring &from, mstring & to);
-mstring UrlDecode(const mstring &from);
-
+mstring UrlEscape(cmstring &s);
+void UrlEscapeAppend(cmstring &s, mstring &sTarget);
+bool UrlUnescapeAppend(cmstring &from, mstring & to);
+// Decode with result as return value, no error reporting
+inline mstring UrlUnescape(cmstring &from)
+{
+	mstring ret; // let the compiler optimize
+	UrlUnescapeAppend(from, ret);
+	return ret;
+}
+mstring DosEscape(cmstring &s);
 
 #define pathTidy(s) { if(startsWithSz(s, "." SZPATHSEP)) s.erase(0, 2); tStrPos n(0); \
 	for(n=0;stmiss!=n;) { n=s.find(SZPATHSEP SZPATHSEP, n); if(stmiss!=n) s.erase(n, 1);}; \
@@ -321,8 +328,9 @@ public:
 	: s(*line), start(0), len(stmiss), oob(line->size()), m_seps(separators) {}
 	inline bool Next()
 	{
-		if(len != stmiss) // initial state
+		if(len != stmiss) // not initial state, find the next position
 			start = start + len + 1;
+
 		if(start>=oob)
 			return false;
 
@@ -347,6 +355,7 @@ public:
 	}
 	inline mstring str(){ return s.substr(start, len); }
 	inline operator mstring() { return str(); }
+//	inline const char *rest() { return s.c_str() + start; }
 };
 
 //bool CreateDetachedThread(void *(*threadfunc)(void *));
@@ -409,7 +418,13 @@ static inline time_t GetTime()
 	return ::time(0);
 }
 
-#define END_OF_TIME (MYSTD::numeric_limits<time_t>::max())
+// arbitrary chosen time declaring some date in far future
+// -1 is preserved for convenience reasons (avoid clashes with time(2) return code and similar uses)
+#ifndef PTHREAD_COND_TIMEDWAIT_TIME_RANGE_OK
+#define END_OF_TIME (time_t(MAX_VAL(int))-2)
+#else
+#define END_OF_TIME (MAX_VAL(time_t)-2)
+#endif
 
 static inline UINT FormatTime(char *buf, const time_t cur)
 {
