@@ -11,7 +11,7 @@
 #include "sockio.h"
 
 class tcpconnect;
-class fileitem_base;
+class fileitem;
 typedef SHARED_PTR<tcpconnect> tTcpHandlePtr;
 
 class tcpconnect
@@ -25,7 +25,7 @@ public:
 	/// Moves the connection handle to the reserve pool (resets the specified sptr).
 	/// Shold only be supplied with IDLE connection handles in a sane state.
 	static void RecycleIdleConnection(tTcpHandlePtr & handle);
-	static time_t ExpireCache();
+	static time_t BackgroundCleanup();
 
 	bool Connect(cmstring &sHostname, cmstring &sPort, mstring &sErrOut);
 	inline int GetFD() { return m_conFd; }
@@ -34,6 +34,8 @@ public:
 	const tHttpUrl * GetProxyData() const { return m_proxy; }
 	void Disconnect();
 
+	static void dump_status();
+
 protected:
 	tcpconnect operator=(const tcpconnect&);
 	tcpconnect(const tcpconnect&);
@@ -46,11 +48,11 @@ protected:
 
 	acfg::tRepoData::IHookHandler *m_pConnStateObserver;
 
-	WEAK_PTR<fileitem_base> m_lastFile;
+	WEAK_PTR<fileitem> m_lastFile;
 
 public:
 	//! @brief Remember the file name belonging to the recently initiated transfer
-	inline void KnowLastFile(WEAK_PTR<fileitem_base> spRemItem) { m_lastFile = spRemItem; }
+	inline void KnowLastFile(WEAK_PTR<fileitem> spRemItem) { m_lastFile = spRemItem; }
 	//! @brief Invalidate (truncate) recently touched file
 	void KillLastFile();
 };
