@@ -43,6 +43,7 @@ string sPopularPath("/debian/");
 string tmpDontcache, tmpDontcacherq, tmpDontcacheResolved;
 
 tStrMap localdirs;
+lockable mimemap_lock;
 NoCaseStringMap mimemap;
 
 MYSTD::bitset<65536> *pUserPorts = NULL;
@@ -113,6 +114,9 @@ MapNameToInt n2iTbl[] = {
 		,{ "RedirMax", &redirmax, NULL, 10}
 		,{ "VfileUseRangeOps", &vrangeops, NULL, 10}
 		,{ "ResponseFreezeDetectTime", &stucksecs, NULL, 10}
+		,{ "ReuseConnections", &persistoutgoing, NULL, 10}
+		,{ "PipelineDepth", &pipelinelen, NULL, 10}
+		,{ "ExSuppressAdminNotification", &exsupcount, NULL, 10}
 
 		,{ "DirPerms", &dirperms, NULL, 8}
 		,{ "FilePerms", &fileperms, NULL, 8}
@@ -405,7 +409,7 @@ struct tHookHandler: public tRepoData::IHookHandler, public lockable
 		{
 			//system(cmdRel.c_str());
 			downTimeNext = ::time(0) + downDuration;
-			g_victor.RunLatestAt(downTimeNext);
+			g_victor.ScheduleFor(downTimeNext, cleaner::TYPE_ACFGHOOKS);
 		}
 	}
 	virtual void JobConnect()
@@ -531,29 +535,61 @@ inline void _ParseLocalDirs(cmstring &value)
 	}
 }
 
-inline void _FetchMimeTypes()
+
+cmstring & GetMimeType(cmstring &path)
 {
-	for(tCfgIter itor("/etc/mime.types");itor.Next();)
 	{
-		// # regular types:
-		// text/plain             asc txt text pot brf  # plain ascii files
-
-		tSplitWalk split(&itor.sLine);
-		if(!split.Next())
-			continue;
-
-		mstring mimetype=split;
-		if(startsWithSz(mimetype, "#"))
-			continue;
-
-		while(split.Next())
+		lockguard g(mimemap_lock);
+		static bool inited = false;
+		if (!inited)
 		{
-			mstring suf=split;
-			if(startsWithSz(suf, "#"))
-				break;
-			mimemap[suf] = mimetype;
+			inited = true;
+			for (tCfgIter itor("/etc/mime.types"); itor.Next();)
+			{
+				// # regular types:
+				// text/plain             asc txt text pot brf  # plain ascii files
+
+				tSplitWalk split(&itor.sLine);
+				if (!split.Next())
+					continue;
+
+				mstring mimetype = split;
+				if (startsWithSz(mimetype, "#"))
+					continue;
+
+				while (split.Next())
+				{
+					mstring suf = split;
+					if (startsWithSz(suf, "#"))
+						break;
+					mimemap[suf] = mimetype;
+				}
+			}
 		}
 	}
+
+	tStrPos dpos = path.find_last_of('.');
+	if (dpos != stmiss)
+	{
+		NoCaseStringMap::const_iterator it = acfg::mimemap.find(path.substr(
+				dpos + 1));
+		if (it != acfg::mimemap.end())
+			return it->second;
+	}
+	// try some educated guess... assume binary if we are sure, text if we are almost sure
+	static cmstring os("application/octet-stream"), tp("text/plain"), nix("");
+	filereader f;
+	if(f.OpenFile(path, true))
+	{
+		size_t maxLen = MYSTD::min(size_t(255), f.GetSize());
+		for(UINT i=0; i< maxLen; ++i)
+		{
+			if(!isascii((UINT) *(f.GetBuffer()+i)))
+				return os;
+		}
+		return tp;
+	}
+	return nix;
 }
 
 bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
@@ -666,7 +702,6 @@ bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
 	else if(0==strcasecmp(key.c_str(), "LocalDirs"))
 	{
 		_ParseLocalDirs(value);
-		_FetchMimeTypes();
 		return !localdirs.empty();
 	}
 	else if(0==strncasecmp(key.c_str(), "Remap-", 6))
@@ -835,7 +870,7 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 		{
 			_FixPostPreSlashes(entry.sPath);
 #ifdef DEBUG
-			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI() <<endl;
+			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI(false) <<endl;
 #endif		
 
 			if(entry.sPort.empty())
@@ -1028,8 +1063,6 @@ void ReadConfigDirectory(const char *szPath)
 
 void PostProcConfig() 
 {
-	if(stupidfs)
-		cerr << "Warning, support for primitive filesystems is not completely implemented!\n";
 	
 	// let's also apply the umask to the directory permissions
 	{
@@ -1078,7 +1111,7 @@ void PostProcConfig()
    }
    
    if(acfg::tpthreadmax < 0)
-	   acfg::tpthreadmax = numeric_limits<int>::max();
+	   acfg::tpthreadmax = MAX_VAL(int);
 	   
    // get rid of duplicated and trailing slash(es)
 	for(tStrPos pos; stmiss != (pos = cachedir.find(SZPATHSEP SZPATHSEP )); )
@@ -1118,6 +1151,9 @@ void PostProcConfig()
    if(RESERVED_DEFVAL == forwardsoap)
 	   forwardsoap = !forcemanaged;
 
+   if(RESERVED_DEFVAL == exsupcount)
+	   exsupcount = (extreshhold >= 5);
+
 #ifdef _SC_NPROCESSORS_ONLN
 	numcores = (int) sysconf(_SC_NPROCESSORS_ONLN);
 #elif defined(_SC_NPROC_ONLN)
@@ -1153,6 +1189,9 @@ void PostProcConfig()
    if(redirmax == RESERVED_DEFVAL)
 	   redirmax = forcemanaged ? 0 : ACFG_REDIRMAX_DEFAULT;
 
+   if(!persistoutgoing)
+	   pipelinelen = 0;
+
    if (acfg::debug >= LOG_MORE)
 	{
 		for (UINT i = 0; i < _countof(n2sTbl); i++)
@@ -1213,14 +1252,15 @@ void PostProcConfig()
 	   */
 	   for(tUrl2RepIter it=mapUrl2pVname.begin(); it!=mapUrl2pVname.end(); it++)
 	   {
-		   cout << it->first.ToURI() << " ___" << *(it->second) << endl;
+		   cout << it->first.ToURI(false) << " ___" << *(it->second) << endl;
 	   }
 
 	   exit(1);
    }
 } // PostProcConfig
 
-time_t ExecutePostponed()
+//! @brief Fires hook callbacks in the background thread
+time_t BackgroundCleanup()
 {
 	time_t ret(END_OF_TIME), now(time(0));
 	for (tRepMap::iterator it = repoparms.begin(); it != repoparms.end(); ++it)
