@@ -30,7 +30,7 @@ string pfilepat(".*(\\.d?deb|\\.rpm|\\.drpm|\\.dsc|\\.tar"COMPRLIST"(\\.gpg)?"
 ")$");
 
 string vfilepat(INFOLDER
-		"(Index|Packages"COMPOPT"|InRelease|Release|Release\\.gpg|"
+		"(Index|Packages"COMPOPT"|InRelease|Release|Release\\.gpg|custom\\.gpg|mirrors.txt|"
 		"Sources"COMPOPT"|release|index\\.db-.*\\.gz|Contents-[^/]*"COMPOPT
 		"|pkglist[^/]*\\.bz2|rclist[^/]*\\.bz2|meta-release[^/]*|Translation[^/]*"COMPOPT
 		"|MD5SUMS|SHA1SUMS" // d-i stuff
@@ -47,7 +47,7 @@ string vfilepat(INFOLDER
 //string wfilepat( VPATPREFIX  "(Release|Release\\.gpg|release|meta-release|Translation[^/]*\\.bz2)$");
 //string wfilepat(vfilepat);
 string wfilepat(INFOLDER
-		"(Release|InRelease|Release\\.gpg"
+		"(Release|InRelease|Release\\.gpg|custom\\.gpg"
 		"|(Packages|Sources)"COMPRLIST"?"
 		"|Translation[^/]*"COMPRLIST"?" // to be checked, but they should never really go anywhere
 		"|MD5SUMS|SHA1SUMS"
@@ -62,9 +62,16 @@ int offlinemode(false), verboselog(true), stupidfs(false), forcemanaged(false),
 extreshhold(20), tpstandbymax(8), tpthreadmax(-1), dirperms(00755), fileperms(00664),
 keepnver(0), maxtempdelay(27), vrangeops(1);
 
-int dnscachetime(1800), dlbufsize(70000), exfailabort(1), exporigin(false), numcores(1),
+int dlbufsize(70000), exfailabort(1), exporigin(false), numcores(1),
 logxff(false), oldupdate(false), recompbz2(false), nettimeout(60), updinterval(0),
-forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL), stucksecs(500);
+forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL),
+stucksecs(500), persistoutgoing(1), pipelinelen(255), exsupcount(RESERVED_DEFVAL);
+
+#ifdef DEBUG
+int dnscachetime(30);
+#else
+int dnscachetime(1800);
+#endif
 
 string agentname("Debian Apt-Cacher-NG/" ACVERSION);
 string remoteport("80"), port("3142");
