@@ -19,6 +19,11 @@
 using namespace MYSTD;
 using namespace SMARTPTR_SPACE;
 
+#define LOG_DECO_START "<html><head><style type=\"text/css\">" \
+".WARNING { color: orange; }\n.ERROR { color: red; }\n" \
+"</style></head><body>"
+#define LOG_DECO_END "</body></html>"
+
 bool bSigTaskAbort=false;
 pthread_mutex_t abortMx=PTHREAD_MUTEX_INITIALIZER;
 
@@ -32,7 +37,10 @@ tWuiBgTask::~tWuiBgTask()
 		m_pTracker->SetEnd(m_reportStream.is_open() ? off_t(m_reportStream.tellp()) : off_t(0));
 
 	if(m_reportStream.is_open())
+	{
+		m_reportStream << LOG_DECO_END;
 		m_reportStream.close();
+	}
 }
 
 // the obligatory definition of static members :-(
@@ -70,7 +78,7 @@ void tWuiBgTask::Run(const string &cmd)
 
 	SendChunkedPageHeader("200");
 
-	acbuf deco;
+	tSS deco;
 	const char *mark(NULL);
 	if(m_szDecoFile &&
 			( deco.initFromFile((acfg::confdir+SZPATHSEP+m_szDecoFile).c_str())
@@ -80,19 +88,21 @@ void tWuiBgTask::Run(const string &cmd)
 		mark=::strchr(deco.rptr(), '~');
 		if(mark)
 		{
-			SendChunk(deco.rptr(), mark-deco.rptr());
+			// send fancy header only to the remote caller
+			SendChunk(deco.rptr(), mark-deco.rptr(), true);
 			deco.drop((mark-deco.rptr())+1);
 		}
 		else
 		{
-			SendChunk(deco.rptr(), deco.size());
+			// send fancy header only to the remote caller
+			SendChunk(deco, true);
 			deco.clear();
 		}
 	}
 
 	tSS msg;
 
-	SMARTPTR_SPACE::shared_ptr<tProgressTracker> pTracked;
+	tProgTrackPtr pTracked;
 
 	{ // this is locked just to make sure that only one can register as master
 		lockguard guard(abortMx);
@@ -102,10 +112,12 @@ void tWuiBgTask::Run(const string &cmd)
 			m_pTracker.reset(new tProgressTracker);
 			msg.clear();
 			time_t nMaintId=time(0);
-			msg<<acfg::logdir<<CPATHSEP<<"maint_" << nMaintId << ".log.html";
+
+			msg<<acfg::logdir<<CPATHSEP<< MAINT_PFX << nMaintId << ".log.html";
 			m_reportStream.open(msg.c_str(), ios::out);
 			if(m_reportStream.is_open())
 			{
+				m_reportStream << LOG_DECO_START;
 				m_reportStream.flush();
 				m_pTracker->id = nMaintId;
 				g_pTracker=m_pTracker;
@@ -120,7 +132,7 @@ void tWuiBgTask::Run(const string &cmd)
 
 	if(pTracked)
 	{
-		msg << "<font color=blue>A maintenance task is already running!</font><br>\n"
+		msg << "<font color=\"blue\">A maintenance task is already running!</font><br>\n"
 				"Attempting to attach to the log output... \n";
 		SendChunk(msg);
 		msg.clear();
@@ -141,7 +153,7 @@ void tWuiBgTask::Run(const string &cmd)
 
 			if(!fd)
 			{
-				msg<<acfg::logdir<<CPATHSEP<<"maint_" << pTracked->id << ".log.html";
+				msg<<acfg::logdir<<CPATHSEP<<MAINT_PFX << pTracked->id << ".log.html";
 				fd=open(msg.c_str(), O_RDONLY);
 				msg.clear();
 				if(fd>=0)
@@ -162,8 +174,10 @@ void tWuiBgTask::Run(const string &cmd)
 			nSent+=msg.length();
 			msg.clear();
 
-			// when file is truncated (disk full), the end size might be unreachable -> let it time out
-			if(!pTracked->wait(33))
+			/* when file is truncated (disk full), the end size might be unreachable
+			 * -> detect the time out
+			 */
+			if(pTracked->wait_until(GetTime()+33, 1))
 				break;
 		}
 		checkforceclose(fd);
@@ -184,36 +198,41 @@ void tWuiBgTask::Run(const string &cmd)
 			else
 				m_sTypeName.insert(0, "(<b>").append("</b>)");
 
-			SendFmt() << "Maintenance task " << m_sTypeName
-					<< ", apt-cacher-ng version: " ACVERSION
-					" (<a href=" << cmd <<
-					"&sigabort>Cancel</a>)<br><form id=\"mainForm\" action=\"#top\">\n";
+			SendFmt << "Maintenance task " << m_sTypeName
+					<< ", apt-cacher-ng version: " ACVERSION;
+			SendFmtRemote << " (<a href=" << cmd <<
+					"&sigabort>Cancel</a>)";
+			SendFmt << "<br>";
+			SendFmtRemote << "<form id=\"mainForm\" action=\"#top\">\n";
 
 			Action(cmd);
 
+			// format the footer for the Web rendering only
 			msg.clear();
 			msg<<"<br>\n<a href=\"/"<<acfg::reportpage<<"\">Return to main page</a>"
 					"</form>";
 
 			if (m_bShowControls)
 			{
-				SendChunk("<hr><b>Action(s):</b> "
+				SendChunk("<br><hr><b>Action(s):</b> "
 					"<input type=\"submit\" name=\"doDelete\""
 					" value=\"Delete selected files\">"
 					"|<button type=\"button\" onclick=\"checkOrUncheck(true);\">Check all</button>"
-					"<button type=\"button\" onclick=\"checkOrUncheck(false);\">Uncheck all</button><hr>");
+					"<button type=\"button\" onclick=\"checkOrUncheck(false);\">Uncheck all</button><br><hr>",
+					true);
 			}
 
 
 
 			_AddFooter(msg);
-			SendChunk(msg);
+			SendChunk(msg, true);
 	}
 
 	finish_action:
 
 	if(!deco.empty())
-		SendChunk(deco.rptr(), deco.size());
+		SendChunk(deco, true);
+
 	EndTransfer();
 
 }
@@ -229,7 +248,7 @@ void tWuiBgTask::tProgressTracker::SetEnd(off_t endSize)
 	notifyAll();
 }
 
-bool tWuiBgTask::CheckAbortCondition()
+bool tWuiBgTask::CheckStopSignal()
 {
 	lockguard g(&abortMx);
 	return bSigTaskAbort;
@@ -243,17 +262,15 @@ void tWuiBgTask::DumpLog(time_t id)
 		return;
 
 	tSS path(acfg::logdir.length()+24);
-	path<<acfg::logdir<<CPATHSEP<<"maint_" << id << ".log.html";
+	path<<acfg::logdir<<CPATHSEP<<MAINT_PFX << id << ".log.html";
 	if (!reader.OpenFile(path))
-		tWUIPage::SendChunk(_SZ2PS("Log not available"));
+		SendChunk(_SZ2PS("Log not available"), true);
 	else
-		tWUIPage::SendChunk(reader.GetBuffer(), reader.GetSize());
+		SendChunk(reader.GetBuffer(), reader.GetSize(), true);
 }
 
-void tWuiBgTask::SendChunk(const char *data, size_t len)
+void tWuiBgTask::AfterSendChunk(const char *data, size_t len)
 {
-	tWUIPage::SendChunk(data, len);
-
 	if(m_reportStream.is_open())
 	{
 		m_reportStream.write(data, len);
@@ -266,13 +283,14 @@ void tWuiBgTask::SendChunk(const char *data, size_t len)
 time_t tWuiBgTask::GetTaskId()
 {
 	lockguard guard(&abortMx);
-	return g_pTracker.expired() ? 0 : g_pTracker.lock()->id;
+	tProgTrackPtr pTracked = g_pTracker.lock();
+	return pTracked ? pTracked->id : 0;
 }
 
 #ifdef DEBUG
 void tBgTester::Action(cmstring &)
 {
-	for (int i = 0; i < 10 && !CheckAbortCondition(); i++, sleep(1))
+	for (int i = 0; i < 10 && !CheckStopSignal(); i++, sleep(1))
 	{
 		char buf[1024];
 		time_t t;
@@ -280,7 +298,7 @@ void tBgTester::Action(cmstring &)
 		t = time(NULL);
 		tmp = localtime(&t);
 		strftime(buf, sizeof(buf), "%c", tmp);
-		SendFmt() << buf << "<br>\n";
+		SendFmt << buf << "<br>\n";
 	}
 }
 #endif // DEBUG
