@@ -28,6 +28,7 @@
 
 using namespace MYSTD;
 
+static cmstring oldStylei18nIdx("/i18n/Index");
 static cmstring diffIdxSfx(".diff/Index");
 static cmstring sPatchBaseRel("_actmp/patch.base");
 static cmstring sPatchResRel("_actmp/patch.result");
@@ -66,10 +67,10 @@ bool tCacheMan::ProcessDirAfter(const string & sPath, const struct stat &)
 }
 
 
-void tCacheMan::AddIFileCandidate(const string &sPathRel)
+bool tCacheMan::AddIFileCandidate(const string &sPathRel)
 {
 	if(sPathRel.empty())
-		return;
+		return false;
 
 	enumIndexType t;
 	if ( (rechecks::FILE_INDEX == rechecks::GetFiletype(sPathRel)
@@ -80,7 +81,10 @@ void tCacheMan::AddIFileCandidate(const string &sPathRel)
 		tIfileAttribs & atts=m_indexFilesRel[sPathRel];
 		atts.vfile_ondisk=true;
 		atts.eIdxType=t;
+
+		return true;
     }
+	return false;
 }
 
 const tCacheMan::tIfileAttribs & tCacheMan::GetFlags(cmstring &sPathRel) const
@@ -164,9 +168,9 @@ bool tCacheMan::FindURL_InitDload(tFileItemPtr fi, cmstring &sFilePathRel,
 					 * */
 					if (m_bVerbose || url.sHost.find(".") != stmiss)
 					{
-						SendFmt()<<"<font color=\"orange\">Code 520824! "
+						SendFmt <<"<span class=\"WARNING\">Code 520824! "
 							"Read the manual about known bugs! Attempting to use "
-								<< url.sHost << " as hostname</font>";
+								<< url.sHost << " as hostname</span>";
 					}
 				}
 			}
@@ -245,8 +249,7 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 {
 	mstring sErr;
 	bool bSuccess=false;
-
-#ifdef DEBUG
+#ifdef DEBUG_FLAGS
 	bool nix=StrHas(sFilePathRel, "debrep/dists/experimental/non-free/debian-installer/binary-mips/Packages.gz");
 #endif
 
@@ -254,25 +257,27 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 	if(flags.uptodate)
 	{
 		if(nVerbosity)
-			SendFmt()<<"Checking "<<sFilePathRel<< (bIndexFile
+			SendFmt<<"Checking "<<sFilePathRel<< (bIndexFile
 					? "... (fresh)<br>\n" : "... (complete)<br>\n");
 		return true;
 	}
 
-#define GOTOREPMSG(x) {sErr = x; bSuccess=false; if(fi) {fi->Unreg(); fi.reset();} goto rep_dlresult; }
+#define GOTOREPMSG(x) {sErr = x; bSuccess=false; goto rep_dlresult; }
 
 	tHttpUrl origURL;
-	tFileItemPtr fi;
+
 	// handle alternative behavior in the first or last cycles
 	if (pForcedURL && !origURL.SetHttpUrl(pForcedURL))
 		return false;
 
-	fi = pItemDelg ? *pItemDelg : fileitem::GetRegisteredFileItem(sFilePathRel, false);
+	fileItemMgmt fi(pItemDelg
+			? pItemDelg->GetFileItemPtr()
+			: fileItemMgmt::GetRegisteredFileItem(sFilePathRel, false));
 
-	if (!fi)
+	if (!fi.get())
 	{
 		if (nVerbosity)
-			SendFmt() << "Checking " << sFilePathRel << "...\n"; // just display the name ASAP
+			SendFmt << "Checking " << sFilePathRel << "...\n"; // just display the name ASAP
 		GOTOREPMSG(" could not create file item handler.");
 	}
 	if (bIndexFile && m_bForceDownload)
@@ -280,62 +285,68 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 		if (!fi->SetupClean())
 			GOTOREPMSG("Item busy, cannot reload");
 		if (nVerbosity)
-			SendFmt() << "Downloading " << sFilePathRel << "...\n";
+			SendFmt << "Downloading " << sFilePathRel << "...\n";
 	}
 	else
 	{
-		FiStatus initState = fi->Setup(bIndexFile);
-		if (FIST_COMPLETE == initState)
+		fileitem::FiStatus initState = fi->Setup(bIndexFile);
+		if (initState > fileitem::FIST_COMPLETE)
+			GOTOREPMSG(fi->GetHeader().frontLine);
+		if (fileitem::FIST_COMPLETE == initState)
 		{
-			SendFmt() << "Checking " << sFilePathRel << "... (complete)<br>\n";
+			int hs = fi->GetHeader().getStatus();
+			if(hs != 200)
+			{
+				SendFmt << "Error downloading " << sFilePathRel << ":\n";
+				goto format_error;
+				//GOTOREPMSG(fi->GetHeader().frontLine);
+			}
+			SendFmt << "Checking " << sFilePathRel << "... (complete)<br>\n";
 			return true;
 		}
-		if (initState > FIST_COMPLETE)
-		{
-			GOTOREPMSG(fi->GetHeader().frontLine);
-		}
 		if (nVerbosity)
-			SendFmt() << (bIndexFile ? "Checking/Updating " : "Downloading ") << sFilePathRel
-					<< "...\n";
+			SendFmt << (bIndexFile ? "Checking/Updating " : "Downloading ")
+			<< sFilePathRel	<< "...\n";
 	}
 
-	if (!FindURL_InitDload(fi, sFilePathRel, sErr, &origURL))
+	if (!FindURL_InitDload(fi.get(), sFilePathRel, sErr, &origURL))
 		GOTOREPMSG("Failed to resolve source URL");
 
 	if (origURL.sHost.empty())
 		GOTOREPMSG("Failed to resolve original URL")
 
-	m_pDlcon->AddJob(fi, origURL);
+	m_pDlcon->AddJob(fi.get(), origURL);
 
 	m_pDlcon->WorkLoop();
-	if (fi->WaitForFinish(NULL) == FIST_COMPLETE)
+	if (fi->WaitForFinish(NULL) == fileitem::FIST_COMPLETE
+			&& fi->GetHeaderUnlocked().getStatus() == 200)
 	{
 		bSuccess = true;
 		if (nVerbosity)
-			SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
-		fi->Unreg(); // anyhow, release our pointer
-		fi.reset();
+			SendFmt << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
 	}
 	else
 	{
+		format_error:
 		if (IsDeprecatedArchFile(sFilePathRel))
 		{
 			if (nVerbosity == VERB_SHOW)
-				SendFmt() << "<i>(no longer available)</i>\n";
-			m_forcedTrashMap[sFilePathRel] = true;
+				SendFmt << "<i>(no longer available)</i>\n";
+			m_forceKeepInTrash[sFilePathRel] = true;
 			bSuccess = true;
 		}
-		else if (flags.forgiveDlErrors)
+		else if (flags.forgiveDlErrors
+				||
+				(fi->GetHeaderUnlocked().getStatus() == 404
+								&& endsWith(sFilePathRel, oldStylei18nIdx))
+		)
 		{
 			bSuccess = true;
 			if (nVerbosity == VERB_SHOW)
-				SendFmt() << "<i>(ignored)</i>\n";
+				SendFmt << "<i>(ignored)</i>\n";
 		}
 		else
 			GOTOREPMSG(fi->GetHttpMsg());
-
-		fi->Unreg(); // anyhow, release our pointer
-		fi.reset();
 	}
 
 	rep_dlresult:
@@ -355,7 +366,7 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 			{
 				if (sErr.empty())
 					sErr = "Download error";
-				SendFmt() << "<font color=\"red\"><b>" << sErr << "</b></font>\n";
+				SendFmt << "<span class=\"ERROR\">" << sErr << "</span>\n";
 			}
 		}
 
@@ -364,7 +375,7 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 	}
 
 	if (nVerbosity != VERB_QUIET)
-		SendChunk("<br>\n");
+		SendChunk("\n<br>\n");
 
 	return bSuccess;
 }
@@ -532,7 +543,7 @@ bool tCacheMan::PatchFile(const string &srcRel,
 		const tFingerprint *verifData)
 {
 	if(m_bVerbose)
-		SendFmt()<< "Patching from " << srcRel << " via " << diffIdxPathRel << "...<br>\n";
+		SendFmt<< "Patching from " << srcRel << " via " << diffIdxPathRel << "...<br>\n";
 
 	string sFinalPatch(CACHE_BASE+"_actmp/combined.diff");
 	if(diffIdxPathRel.length()<=diffIdxSfx.length())
@@ -546,11 +557,11 @@ bool tCacheMan::PatchFile(const string &srcRel,
 		if(!Download(pfile, false, VERB_SHOW))
 		{
 			m_indexFilesRel.erase(pfile); // remove the mess for sure
-			SendFmt() << "Failed to download patch file " << pfile << " , stop patching...<br>";
+			SendFmt << "Failed to download patch file " << pfile << " , stop patching...<br>";
 			return false;
 		}
 
-		if(CheckAbortCondition())
+		if(CheckStopSignal())
 			return false;
 
 		SetFlags(pfile).parseignore=true; // not an ifile, never parse this
@@ -564,16 +575,16 @@ bool tCacheMan::PatchFile(const string &srcRel,
 		if(!probe.ScanFile(CACHE_BASE+pfile, CSTYPE_SHA1, true, pf.p))
 		{
 
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return false;
 
 			if(m_bVerbose)
-				SendFmt() << "Failure on checking of intermediate patch data in " << pfile << ", stop patching...<br>";
+				SendFmt << "Failure on checking of intermediate patch data in " << pfile << ", stop patching...<br>";
 			return false;
 		}
 		if ( ! (probe == pit->fprPatch))
 		{
-			SendFmt()<< "Bad patch data in " << pfile <<" , stop patching...<br>";
+			SendFmt<< "Bad patch data in " << pfile <<" , stop patching...<br>";
 			return false;
 		}
 	}
@@ -617,6 +628,8 @@ bool tCacheMan::PatchFile(const string &srcRel,
 	return true;
 
 }
+
+// dirty little helper used to store the head and only the head to a specified location
 class fakeCon : public ::tCacheMan::tGetItemDelegate
 {
    public:
@@ -626,7 +639,7 @@ class fakeCon : public ::tCacheMan::tGetItemDelegate
 			sTempDataRel(a), sReferencePathRel(b), nGotSize(-1)
 	{
 	}
-	operator tFileItemPtr()
+	fileItemMgmt GetFileItemPtr()
 	{
 		class tHeadOnlyStorage: public fileitem_with_storage
 		{
@@ -640,13 +653,15 @@ class fakeCon : public ::tCacheMan::tGetItemDelegate
 				m_bHeadOnly = true;
 				m_head.LoadFromFile(SABSPATH(p.sReferencePathRel+".head"));
 			}
-			void Unreg()
+			~tHeadOnlyStorage()
 			{
-				m_head.StoreToFile(m_sPathAbs + ".head");
+				m_head.StoreToFile( CACHE_BASE + m_sPathRel + ".head");
 				m_parent.nGotSize = atoofft(m_head.h[header::CONTENT_LENGTH], -17);
 			}
 		};
-		return tFileItemPtr(static_cast<fileitem_base*>(new tHeadOnlyStorage(*this)));
+		fileItemMgmt ret;
+		ret.ReplaceWithLocal(static_cast<fileitem*>(new tHeadOnlyStorage(*this)));
+		return ret;
 	}
 	virtual ~fakeCon() {}
 };
@@ -669,9 +684,9 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 	if(GetFlags(to).uptodate)
 		return true;
 
-#ifdef DEBUG
+#ifdef DEBUG_FLAGS
 	bool nix = stmiss!=from.find("debrep/dists/squeeze/non-free/binary-amd64/");
-	SendFmt()<<"Replacing "<<to<<" with " << from <<  "<br>\n";
+	SendFmt<<"Replacing "<<to<<" with " << from <<  "<br>\n";
 #endif
 
 	header head;
@@ -692,30 +707,31 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 		}
 	}
 
-	class tInjectItem : public fileitem
+	class tInjectItem : public fileitem_with_storage
 	{
 	public:
 		bool m_link;
-		tInjectItem(cmstring &to, bool bTryLink) : fileitem(to), m_link(bTryLink)
+		tInjectItem(cmstring &to, bool bTryLink) : fileitem_with_storage(to), m_link(bTryLink)
 		{
 		}
 		// noone else should attempt to store file through it
-		virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &, const char *,
-				bool)
+		virtual bool DownloadStartedStoreHeader(const header &, const char *,
+				bool, bool*)
 		{
 			return false;
 		}
 		virtual bool Inject(cmstring &fromRel, const header &head)
 		{
 			m_head = head;
-			mkbasedir(m_sPathAbs);
-			if(head.StoreToFile(m_sPathAbs+".head")<=0)
+			mstring sPathAbs = SABSPATH(m_sPathRel);
+			mkbasedir(sPathAbs);
+			if(head.StoreToFile(sPathAbs+".head") <=0)
 				return false;
 
 			if(m_link)
 			{
 				setLockGuard;
-				if (LinkOrCopy(SABSPATH(fromRel), m_sPathAbs))
+				if (LinkOrCopy(SABSPATH(fromRel), sPathAbs))
 				{
 					m_status = FIST_COMPLETE;
 					notifyAll();
@@ -737,9 +753,10 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 			}
 			*/
 
-			if (Setup(true) >= FIST_LOCALERROR)
+			if (Setup(true) > fileitem::FIST_COMPLETE)
 				return false;
-			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, NULL, false))
+			bool bNix(false);
+			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, NULL, false, bNix))
 				return false;
 			if(!StoreFileData(data.GetBuffer(), data.GetSize()) || ! StoreFileData(NULL, 0))
 				return false;
@@ -749,8 +766,10 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 		}
 	};
 	tInjectItem *p=new tInjectItem(to, bTryLink);
-	SHARED_PTR<fileitem> fi(static_cast<fileitem*>(p));
-	if (!fi || !fileitem::RegisterFileItem(fi) )
+	tFileItemPtr pfi(static_cast<fileitem*>(p));
+	// register it in global scope
+	fileItemMgmt fi(fileItemMgmt::RegisterFileItem(pfi));
+	if (!fi)
 	{
 		MTLOGASSERT(false, "Couldn't register copy item");
 		return false;
@@ -760,8 +779,6 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 	if(bOK && bUpdateRefdata)
 		UpdateFingerprint(to, -1, NULL, NULL);
 
-	fi->Unreg();
-
 	MTLOGASSERT(bOK, "Inject: failed");
 
 	if(bSetIfileFlags)
@@ -777,41 +794,22 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassIter,
 		cmstring *psTmpUnpackedAbs)
 {
-#ifdef DEBUG
-	SendFmt()<< "Installing " << donorRel << "<br>\n";
+#ifdef DEBUG_FLAGS
+	SendFmt<< "Installing " << donorRel << "<br>\n";
 	bool nix=StrHas(donorRel, "debrep/dists/experimental/main/binary-amd64/Packages");
 #endif
 
 	const tStrDeq &tgts = eqClassIter->second.paths;
 
-	// we know it's uptodate, make sure that attempts to modify it in background
-	// it also needs to be reliable disarmed before releasing, use RAII for that
-	struct keeper
+	// we know it's uptodate, make sure to avoid attempts to modify it in background
+	fileItemMgmt src;
+	if(GetFlags(donorRel).uptodate)
 	{
-		tFileItemPtr pItem;
-		keeper(const string &path, bool y) { if(y) pItem=fileitem::GetRegisteredFileItem(path, false); }
-		~keeper() { if(pItem) pItem->Unreg(); }
-	} src(donorRel, GetFlags(donorRel).uptodate);
-
-	//tStrPos cpos=FindComPos(donor);
-
-	// make sure that the source file has the .head file and has the final size
-	if(src.pItem)
-	{
-		src.pItem->Setup(false);
+		src = fileItemMgmt::GetRegisteredFileItem(donorRel, false);
+		src->Setup(false);
 		// something changed it in meantime?!
-		if(src.pItem->GetStatus() != FIST_COMPLETE)
+		if (src->GetStatus() != fileitem::FIST_COMPLETE)
 			return false;
-
-		/* Disable the double-check, doesn't work well when compressed files serves as content id
-		if(stmiss == cpos)
-		{
-			off_t nCheckSize(0); // just to be sure
-			src.pItem->GetStatusUnlocked(nCheckSize);
-			if(eqClassIter->first.first.size != nCheckSize) // hm?
-				return false;
-		}
-		*/
 	}
 
 	int nInjCount=0;
@@ -971,7 +969,7 @@ void tCacheMan::UpdateIndexFiles()
 		{
 			m_nErrorCount+=(!m_indexFilesRel[sPathRel].hideDlErrors);
 
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return;
 
 			continue;
@@ -1027,19 +1025,19 @@ void tCacheMan::UpdateIndexFiles()
 	/*
 	for(tContId2eqClass::iterator it=m_eqClasses.begin(); it!=m_eqClasses.end();it++)
 	{
-		SendFmt()<<"__TID: " << it->first.first<<it->first.second<<"<br>"
+		SendFmt<<"__TID: " << it->first.first<<it->first.second<<"<br>"
 				<< "bz2TID:" << it->second.bz2VersContId.first<< it->second.bz2VersContId.second<<"<br>"
 				<< "idxTID:"<<it->second.diffIdxId.first << it->second.diffIdxId.second <<"<br>"
 				<< "Paths:<br>";
 		for(tStrDeq::const_iterator its=it->second.paths.begin();
 				its!=it->second.paths.end(); its++)
 		{
-			SendFmt()<<"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" << *its<<"<br>";
+			SendFmt<<"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" << *its<<"<br>";
 		}
 	}
 	*/
 
-	if(CheckAbortCondition())
+	if(CheckStopSignal())
 		return;
 
 	/*
@@ -1083,14 +1081,14 @@ void tCacheMan::UpdateIndexFiles()
 #ifdef DEBUG
 	for(tContId2eqClass::iterator it=m_eqClasses.begin(); it!=m_eqClasses.end();it++)
 	{
-		SendFmt()<<"TID: " << it->first.first<<it->first.second<<"<br>"
+		SendFmt<<"TID: " << it->first.first<<it->first.second<<"<br>"
 				<< "bz2TID:" << it->second.bz2VersContId.first<< it->second.bz2VersContId.second<<"<br>"
 				<< "idxTID:"<<it->second.diffIdxId.first << it->second.diffIdxId.second <<"<br>"
 				<< "Paths:<br>";
 		for(tStrDeq::const_iterator its=it->second.paths.begin();
 				its!=it->second.paths.end(); its++)
 		{
-			SendFmt()<<"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" << *its<<"<br>";
+			SendFmt<<"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" << *its<<"<br>";
 		}
 	}
 	SendChunk("<br><br><b>FOLLOWING VOID WHICH BINDS</b><br><br>");
@@ -1099,10 +1097,10 @@ void tCacheMan::UpdateIndexFiles()
 	DelTree(SABSPATH("_actmp")); // do one to test the permissions
 	/* wrong check but ignore for now
 	if(::access(SZABSPATH("_actmp"), F_OK))
-		SendFmt()
-		<< "<font color=\"orange\">Warning, failed to purge temporary directory "
+		SendFmt
+		<< "<span class=\"WARNING\">Warning, failed to purge temporary directory "
 		<< CACHE_BASE << "_actmp/, this could impair some additional functionality"
-				"</font><br>";
+				"</span><br>";
 */
 
 	// Iterate over classes and do patch-update where possible
@@ -1115,7 +1113,7 @@ void tCacheMan::UpdateIndexFiles()
 		tFingerprint *pEndSum(NULL);
 		tPListConstIt itPatchStart;
 
-		if(CheckAbortCondition())
+		if(CheckStopSignal())
 			return;
 
 		DelTree(SABSPATH("_actmp"));
@@ -1140,7 +1138,7 @@ void tCacheMan::UpdateIndexFiles()
 		if (!Download(patchidxFileToUse, true, VERB_SHOW))
 			continue;
 
-		if(CheckAbortCondition())
+		if(CheckStopSignal())
 			return;
 
 		pEndSum=BuildPatchList(CACHE_BASE+patchidxFileToUse, patchList);
@@ -1152,7 +1150,7 @@ void tCacheMan::UpdateIndexFiles()
 		for(deque<tPatchEntry>::const_iterator itPinfo = patchList.begin();
 				pEndSum && itPinfo!=patchList.end(); ++itPinfo)
 		{
-			SendFmt() << itPinfo->patchName<< " -- " << itPinfo->fprState
+			SendFmt << itPinfo->patchName<< " -- " << itPinfo->fprState
 					<<" / " << itPinfo->fprPatch<<  " <br>";
 		}
 		*/
@@ -1168,7 +1166,7 @@ void tCacheMan::UpdateIndexFiles()
 			df.p = fopen(sPatchBaseAbs.c_str(), "w");
 			if(!df.p)
 			{
-				SendFmt() << "Cannot write temporary patch data to " << sPatchBaseAbs << "<br>";
+				SendFmt << "Cannot write temporary patch data to " << sPatchBaseAbs << "<br>";
 				break;
 			}
 			if (GetFlags(*its).vfile_ondisk)
@@ -1185,7 +1183,7 @@ void tCacheMan::UpdateIndexFiles()
 				{
 					df.close(); // write the whole file to disk ASAP!
 
-					if(CheckAbortCondition())
+					if(CheckStopSignal())
 						return;
 
 					// Hit the current state, no patching needed for it?
@@ -1194,11 +1192,11 @@ void tCacheMan::UpdateIndexFiles()
 						// since we know the stuff is fresh, no need to refetch it later
 						m_indexFilesRel[*its].uptodate=true;
 						if(m_bVerbose)
-							SendFmt() << "Found fresh version in " << *its << "<br>";
+							SendFmt << "Found fresh version in " << *its << "<br>";
 
 						Propagate(*its, cid2eqcl, &sPatchBaseAbs);
 
-						if(CheckAbortCondition())
+						if(CheckStopSignal())
 							return;
 
 						goto CONTINUE_NEXT_GROUP;
@@ -1215,23 +1213,23 @@ void tCacheMan::UpdateIndexFiles()
 							h.set(header::XORIG, s.substr(0, FindComPos(s)));
 						}
 
-						if(CheckAbortCondition())
+						if(CheckStopSignal())
 							return;
 
 						if (m_bVerbose)
-							SendFmt() << "Found patching base candidate, unpacked to "
+							SendFmt << "Found patching base candidate, unpacked to "
 							<< sPatchBaseAbs << "<br>";
 
 						if (PatchFile(sPatchBaseAbs, patchidxFileToUse, itPatchStart,
 								patchList.end(), pEndSum))
 						{
 
-							if(CheckAbortCondition())
+							if(CheckStopSignal())
 								return;
 
 							h.set(header::LAST_MODIFIED, FAKEDATEMARK);
 							h.set(header::CONTENT_LENGTH, pEndSum->size);
-							if (h.StoreToFile(SABSPATH(sPatchResRel) + ".head") <= 0)
+							if (h.StoreToFile(CACHE_BASE+sPatchResRel+".head") <= 0)
 							{
 								MTLOGDEBUG("############ Failed to store target header as "
 										<< SABSPATH(sPatchResRel) << ".head");
@@ -1241,12 +1239,12 @@ void tCacheMan::UpdateIndexFiles()
 							SendChunk("Patching result: succeeded<br>");
 							Propagate(sPatchResRel, cid2eqcl);
 
-							if(CheckAbortCondition())
+							if(CheckStopSignal())
 								return;
 
 							InstallBz2edPatchResult(cid2eqcl);
 
-							if(CheckAbortCondition())
+							if(CheckStopSignal())
 								return;
 
 							break;
@@ -1257,7 +1255,7 @@ void tCacheMan::UpdateIndexFiles()
 							// don't break, maybe the next one can be patched
 						}
 
-						if(CheckAbortCondition())
+						if(CheckStopSignal())
 							return;
 					}
 				}
@@ -1268,7 +1266,7 @@ void tCacheMan::UpdateIndexFiles()
 		NOT_PATCHABLE:
 		/*
 		if(m_bVerbose)
-			SendFmt() << "Cannot update " << it->first << " by patching, what next?"<<"<br>";
+			SendFmt << "Cannot update " << it->first << " by patching, what next?"<<"<br>";
 */
 		// prefer to download them in that order, no uncompressed versions because
 		// mirrors usually don't have them
@@ -1283,20 +1281,20 @@ void tCacheMan::UpdateIndexFiles()
 					continue;
 				if(Download(cand, true, GetFlags(cand).hideDlErrors ? VERB_SHOW_NOERRORS : VERB_SHOW))
 				{
-					if(CheckAbortCondition())
+					if(CheckStopSignal())
 						return;
 					if(Propagate(cand, cid2eqcl)) // all locations are covered?
 						goto CONTINUE_NEXT_GROUP;
 				}
 
-				if(CheckAbortCondition())
+				if(CheckStopSignal())
 					return;
 			}
 		}
 
 		CONTINUE_NEXT_GROUP:
 
-		if(CheckAbortCondition())
+		if(CheckStopSignal())
 			return;
 	}
 
@@ -1363,7 +1361,7 @@ void tCacheMan::InstallBz2edPatchResult(tContId2eqClass::iterator eqClassIter)
 			&& GetAndCheckHead(sFreshBz2Rel, sRefBz2Rel, bz2fpr.size))
 	{
 		if (m_bVerbose)
-			SendFmt() << "Compressed into " << sFreshBz2Rel << "<br>\n";
+			SendFmt << "Compressed into " << sFreshBz2Rel << "<br>\n";
 	}
 	else
 		return;
@@ -1371,13 +1369,13 @@ void tCacheMan::InstallBz2edPatchResult(tContId2eqClass::iterator eqClassIter)
 	inject_bz2s:
 	// use a recursive call to distribute bz2 versions
 
-	if(CheckAbortCondition())
+	if(CheckStopSignal())
 		return;
 
 	if (!sFreshBz2Rel.empty())
 	{
 #ifdef DEBUG
-		SendFmt() << "Recursive call to install the bz2 version from " << sFreshBz2Rel << "<br>";
+		SendFmt << "Recursive call to install the bz2 version from " << sFreshBz2Rel << "<br>";
 #endif
 		Propagate(sFreshBz2Rel, eqClassIter);
 	}
@@ -1427,7 +1425,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		enumIndexType idxType)
 {
 
-#ifdef DEBUG
+#ifdef DEBUG_FLAGS
 	bool bNix=StrHas(sPath, "/i18n/");
 #endif
 
@@ -1450,7 +1448,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 	else
 	{
 		m_nErrorCount++;
-		SendFmt() << "Unexpected index file without subdir found: " << sPath;
+		SendFmt << "Unexpected index file without subdir found: " << sPath;
 		return false;
 	}
 
@@ -1463,8 +1461,8 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		if(! GetFlags(sPath).forgiveDlErrors) // that would be ok (added by ignorelist), don't bother
 		{
 			aclog::errnoFmter err;
-			SendFmt()<<"<font color=orange>WARNING: unable to open "<<sPath
-					<<"(" << err << ")</font><br>\n";
+			SendFmt<<"<span class=\"WARNING\">WARNING: unable to open "<<sPath
+					<<"(" << err << ")</span>\n<br>\n";
 		}
 		return false;
 	}
@@ -1489,6 +1487,8 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		LOG("filetype: Packages file");
 		static const string sMD5sum("MD5sum"), sFilename("Filename"), sSize("Size");
 
+		UrlUnescape(sPkgBaseDir);
+
 		while(reader.GetOneLine(sLine))
 		{
 			trimBack(sLine);
@@ -1499,7 +1499,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 					ret.HandlePkgEntry(info, false);
 				info.SetInvalid();
 
-				if(CheckAbortCondition())
+				if(CheckStopSignal())
 					return true; // XXX: should be rechecked by the caller ASAP!
 
 				continue;
@@ -1513,6 +1513,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 					info.fpr.size=atoofft(val.c_str());
 				else if(key==sFilename)
 				{
+					UrlUnescape(val);
 					info.sDirectory=sPkgBaseDir;
 					tStrPos pos=val.rfind(SZPATHSEPUNIX);
 					if(pos==stmiss)
@@ -1546,7 +1547,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 					info.SetInvalid();
 					nStep = 0;
 
-					if (CheckAbortCondition())
+					if (CheckStopSignal())
 						return true;
 
 					continue;
@@ -1588,7 +1589,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 
 		while (reader.GetOneLine(sLine))
 		{
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return true;
 
 			static const string cygkeys[]={"install: ", "source: "};
@@ -1623,7 +1624,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		LOG("SUSE index file, entry level");
 		while(reader.GetOneLine(sLine))
 		{
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return true;
 
 			for(tSplitWalk split(&sLine, "\"'><=/"); split.Next(); )
@@ -1644,7 +1645,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		LOG("XML based file list, pickup any valid filename ending in .rpm");
 		while(reader.GetOneLine(sLine))
 		{
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return true;
 
 			for(tSplitWalk split(&sLine, "\"'><=/"); split.Next(); )
@@ -1698,6 +1699,8 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		tStrDeq fileList;
 		mstring sDirHeader;
 
+		UrlUnescape(sPkgBaseDir);
+
 		while(reader.GetOneLine(sLine))
 		{
 			//if(sLine.find("unp_")!=stmiss)
@@ -1724,13 +1727,17 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 				for(tStrDeq::iterator it=fileList.begin(); it!=fileList.end(); ++it)
 				{
 				// ok, read "checksum size filename" into info and check the word count
+
 				tSplitWalk split(&*it);
+				info.sFileName.clear();
 				if(!split.Next()
 						|| (key = split, !split.Next())
 						|| (val = split, !split.Next())
-						|| (info.sFileName=split, split.Next())
+						|| (UrlUnescapeAppend(split, info.sFileName) && split.Next())
 						|| !info.fpr.Set(key, info.fpr.csType, atoofft(val.c_str())))
+				{
 					continue;
+				}
 
 				switch(origIdxType)
 					{
@@ -1767,7 +1774,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 				fileList.clear();
 			}
 
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return true;
 
 		}
@@ -1775,7 +1782,8 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 	break;
 
 	default:
-		SendChunk("<font color=orange>WARNING: unable to read this file (unsupported format)</font><br>\n");
+		SendChunk("<span class=\"WARNING\">"
+				"WARNING: unable to read this file (unsupported format)</span>\n<br>\n");
 		return false;
 	}
 	return reader.CheckGoodState(false);
@@ -1786,7 +1794,7 @@ void tCacheMan::_ProcessSeenIndexFiles()
 	LOGSTART("expiration::_ParseVolatileFilesAndHandleEntries");
 	for(tS2IDX::const_iterator it=m_indexFilesRel.begin(); it!=m_indexFilesRel.end(); it++)
 	{
-		if(CheckAbortCondition())
+		if(CheckStopSignal())
 			return;
 
 		const tIfileAttribs &att=it->second;
@@ -1823,7 +1831,7 @@ void tCacheMan::_ProcessSeenIndexFiles()
 			if(!m_indexFilesRel[it->first].forgiveDlErrors)
 			{
 				m_nErrorCount++;
-				SendChunk("<font color=red>An error occured while reading this file, some contents may have been ignored.</font>\n");
+				SendChunk("<span class=\"ERROR\">An error occured while reading this file, some contents may have been ignored.</span>\n");
 				AddDelCbox(it->first);
 				SendChunk("<br>\n");
 			}
@@ -1849,13 +1857,13 @@ void tCacheMan::AddDelCbox(cmstring &sFileRel)
 		return;
 
 	m_bShowControls=true;
-	SendFmt() << "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
+	SendFmtRemote <<  "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
 			<< "\" value=\"" << sFileRel << "\">Tag</label>";
 
 }
 void tCacheMan::TellCount(uint nCount, off_t nSize)
 {
-	SendFmt() << "<br>\n" << nCount <<" package file(s) marked "
+	SendFmt << "<br>\n" << nCount <<" package file(s) marked "
 			"for removal in few days. Estimated disk space to be released: "
 			<< offttosH(nSize) << ".<br>\n<br>\n";
 }
@@ -1874,29 +1882,29 @@ void tCacheMan::SetCommonUserFlags(cmstring &cmd)
 
 void tCacheMan::PrintStats(cmstring &title)
 {
-	multimap<off_t, string> sorted;
+	multimap<off_t, cmstring*> sorted;
 	off_t total=0;
+	const UINT nMax = m_bVerbose ? (UINT_MAX-1) : 10;
 	for(tS2IDX::const_iterator it=m_indexFilesRel.begin(); it!=m_indexFilesRel.end(); ++it)
 	{
 		if(it->second.space)
 		{
-			sorted.insert(make_pair(it->second.space, it->first));
+			sorted.insert(make_pair(it->second.space, &(it->first)));
 			total += it->second.space;
 		}
+		if(sorted.size()>nMax)
+			sorted.erase(sorted.begin());
 	}
 	if(!total)
 		return;
+	SendFmt << "<br>\n<table><tr><td colspan=2><u>" << title;
 	if(!m_bVerbose)
-		while(sorted.size() > 10)
-			sorted.erase(sorted.begin());
-	SendFmt() << "<br>\n<table><tr><td colspan=2><u>" << title;
-	if(!m_bVerbose)
-		SendFmt() << " (Top " << sorted.size() << ")";
-	SendFmt() << "</u></td></tr>\n";
-	for(multimap<off_t, string>::reverse_iterator it=sorted.rbegin(); it!=sorted.rend(); ++it)
+		SendFmt << " (Top " << sorted.size() << ")";
+	SendFmt << "</u></td></tr>\n";
+	for(multimap<off_t, cmstring*>::reverse_iterator it=sorted.rbegin(); it!=sorted.rend(); ++it)
 	{
-		SendFmt() << "<tr><td><b>" << offttosH(it->first) << "</b></td><td>"
-				<< it->second << "</td></tr>\n";
+		SendFmt << "<tr><td><b>" << offttosH(it->first) << "</b></td><td>"
+				<< *(it->second) << "</td></tr>\n";
 	}
-	SendFmt() << "</table>";
+	SendFmt << "</table>";
 }
