@@ -40,7 +40,7 @@ con::con(int fdId, const char *c) :
 
 con::~con() {
 	LOGSTART("con::~con (Destroying connection...)");
-	flushShutdownClose(m_confd);
+	termsocket(m_confd);
 
 	// our user's connection is released but the downloader task created here may still be serving others
 	// tell it to stop when it gets the chance and delete it then
@@ -223,7 +223,7 @@ void con::WorkLoop() {
         
         if(ready == 0)
         {
-        	USRDBG(5, "Timeout occurred, apt client disappeared silently?");
+        	USRDBG("Timeout occurred, apt client disappeared silently?");
         	return;
         }
 		else if (ready<0)
@@ -245,7 +245,10 @@ void con::WorkLoop() {
               if(n==-EAGAIN)
                 continue;
               else
-                return;
+              {
+            	  ldbg("client closed connection");
+            	  return;
+              }
             }
         }
 
