@@ -18,6 +18,7 @@
 #include <list>
 #include <map>
 #include <iostream>
+#include <algorithm>    // std::min_element, std::max_element
 
 #ifdef HAVE_LIBWRAP
 #include <tcpd.h>
@@ -131,7 +132,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 	if(!szClientName)
 		szClientName="";
 	
-	USRDBG(1, "Client name: " << szClientName);
+	USRDBG( "Client name: " << szClientName);
 	con *c(NULL);
 
 	{
@@ -142,7 +143,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 		// DOS prevention
 		if (g_freshConQueue.size() > MAX_BACKLOG)
 		{
-			USRDBG(4, "Worker queue overrun");
+			USRDBG( "Worker queue overrun");
 			goto local_con_failure;
 		}
 
@@ -152,7 +153,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 			if (!c)
 			{
 #ifdef NO_EXCEPTIONS
-				USRDBG(4, "Out of memory");
+				USRDBG( "Out of memory");
 #endif
 				goto local_con_failure;
 			}
@@ -163,7 +164,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 
 		} catch (MYSTD::bad_alloc&)
 		{
-			USRDBG(4, "Out of memory");
+			USRDBG( "Out of memory");
 			goto local_con_failure;
 		}
 	}
@@ -171,7 +172,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 	if (!SpawnThreadsAsNeeded())
 	{
 		aclog::errnoFmter fer("Cannot start threads, cleaning up. Reason: ");
-		USRDBG(nan, fer);
+		USRDBG(fer);
 		lockguard g(g_ThreadPoolCondition);
 		while(!g_freshConQueue.empty())
 		{
@@ -185,8 +186,8 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 	local_con_failure:
 	if (c)
 		delete c;
-	USRDBG(3, "Connection setup error");
-	forceShutdownClose(fd);
+	USRDBG( "Connection setup error");
+	termsocket_quick(fd);
 }
 
 void CreateUnixSocket() {
@@ -327,7 +328,7 @@ void Setup()
 		}
 		if(g_vecSocks.empty())
 		{
-			cerr << "No socket(s) could be create/prepared. "
+			cerr << "No socket(s) could be created/prepared. "
 			"Check the network, check or unset the BindAddress directive.\n";
 			exit(EXIT_FAILURE);
 		}
@@ -345,12 +346,8 @@ int Run()
 {
 	LOGSTART2s("Run", "GoGoGo");
 	fd_set rfds, wfds;
-	int maxfd(0);
-	for(vector<int>::iterator it=g_vecSocks.begin(); it!=g_vecSocks.end(); it++)
-		if(*it > maxfd)
-			maxfd=*it;
-	maxfd++;
-	USRDBG(1, "Listening to incoming connections...");
+	int maxfd = 1 + *max_element(g_vecSocks.begin(), g_vecSocks.end());
+	USRDBG( "Listening to incoming connections...");
 
 	while (1)
 	{ // main accept() loop
@@ -382,7 +379,7 @@ int Run()
 				if (fd>=0)
 				{
 					set_nb(fd);
-					USRDBG(1, "Detected incoming connection from the UNIX socket");
+					USRDBG( "Detected incoming connection from the UNIX socket");
 					SetupConAndGo(fd);
 				}
 				else if(errno == EMFILE || errno == ENOMEM || ENOBUFS == errno)
@@ -402,13 +399,13 @@ int Run()
 				{
 					set_nb(fd);
 					char hbuf[NI_MAXHOST];
-					USRDBG(1, "Detected incoming connection from the TCP socket");
+					USRDBG( "Detected incoming connection from the TCP socket");
 
 					if (getnameinfo((struct sockaddr*) &addr, addrlen, hbuf, sizeof(hbuf),
 									NULL, 0, NI_NUMERICHOST))
 					{
 						aclog::err("ERROR: could not resolve hostname for incoming TCP host");
-						forceShutdownClose(fd);
+						termsocket_quick(fd);
 						sleep(1);
 						continue;
 					}
@@ -423,7 +420,7 @@ int Run()
 						if (!hosts_access(&req))
 						{
 							aclog::err("ERROR: access not permitted by hosts files", hbuf);
-							forceShutdownClose(fd);
+							termsocket_quick(fd);
 							continue;
 						}
 #else
@@ -462,7 +459,7 @@ void Shutdown()
 	
 	printf("Closing listening sockets\n");
 	for(vector<int>::iterator it=g_vecSocks.begin(); it!=g_vecSocks.end(); it++)
-		forceShutdownClose(*it);
+		termsocket_quick(*it);
 }
 
 }
