@@ -17,7 +17,7 @@ using namespace MYSTD;
 static const cmstring sGenericError("567 Unknown download error occured");
 
 dlcon::dlcon(bool bManualExecution, string *xff) :
-		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nDisablePling(0)
+		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nTempPipelineDisable(0)
 {
 	LOGSTART("dlcon::dlcon");
 	m_wakepipe[0] = m_wakepipe[1] = -1;
@@ -36,7 +36,7 @@ struct tDlJob
 	mstring sErrorMsg;
 	inline bool HasBrokenStorage()
 	{
-		return (!m_pStorage || m_pStorage->GetStatus() >= FIST_LOCALERROR);
+		return (!m_pStorage || m_pStorage->GetStatus() > fileitem::FIST_COMPLETE);
 	}
 
 #define HINT_MORE 0
@@ -106,15 +106,16 @@ struct tDlJob
 
 	inline tDlJob(dlcon *p, tFileItemPtr pFi, const acfg::tRepoData * pBackends,
 			const MYSTD::string & sPath, int redirmax) :
-			m_pStorage(pFi), m_pBEdata(pBackends), m_fileUri("", sPath),
+			m_pStorage(pFi), m_pBEdata(pBackends),
 			m_nRedirRemaining(redirmax)
 	{
 		Init();
+		m_fileUri.sPath=sPath;
 	}
 
 	~tDlJob()
 	{
-		if (m_pStorage && FIST_COMPLETE != m_pStorage->GetStatus())
+		if (m_pStorage)
 			m_pStorage->DecDlRefCount(sErrorMsg.empty() ? sGenericError : sErrorMsg);
 	}
 
@@ -181,18 +182,25 @@ struct tDlJob
 		if (!proxied) // use absolute path
 		{
 			if (m_pCurBackend) // base dir from backend definition
-				head << m_pCurBackend->sPath;
-			head << m_fileUri.sPath;
+				head << UrlEscape(m_pCurBackend->sPath);
+			head << UrlEscape(m_fileUri.sPath);
 		}
 		else
-			head << RemoteUri();
+			head << RemoteUri(true);
+
+		ldbg(RemoteUri(true));
 
 		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerName() << "\r\n";
 
 		if (proxied) // proxy stuff, and add authorization if there is any
 		{
 			ldbg("using proxy");
-			head << acfg::proxy_info.sPath << "Proxy-Connection: keep-alive\r\n";
+			head << acfg::proxy_info.sPath
+					// Proxy-Connection is a non-sensical copy of Connection but some proxy
+					// might listen only to this one so better add it
+					<< (acfg::persistoutgoing ? "Proxy-Connection: keep-alive\r\n"
+									: "Proxy-Connection: close\r\n");
+
 		}
 
 		// either by backend or by host in file uri, never both
@@ -264,18 +272,20 @@ struct tDlJob
 		if (acfg::exporigin && !xff.empty())
 			head << "X-Forwarded-For: " << xff << "\r\n";
 
-		head << acfg::requestapx << "Accept: */*\r\nConnection: Keep-Alive\r\n\r\n";
+		head << acfg::requestapx << "Accept: */*\r\nConnection: "
+				<< (acfg::persistoutgoing ? "keep-alive\r\n\r\n" : "close\r\n\r\n");
 
-		ldbg("Request cooked, buffer contents: " << head);
 #ifdef SPAM
 		//head.syswrite(2);
 #endif
 
 	}
 
-	inline string RemoteUri()
+	inline string RemoteUri(bool bEscaped)
 	{
-		return (m_pCurBackend ? m_pCurBackend->ToURI() + m_fileUri.sPath : m_fileUri.ToURI());
+		return m_pCurBackend
+				? m_pCurBackend->ToURI(bEscaped) + m_fileUri.sPath
+						: m_fileUri.ToURI(bEscaped);
 	}
 
 	uint_fast8_t NewDataHandler(acbuf & inBuf)
@@ -290,6 +300,7 @@ struct tDlJob
 
 			if (!m_pStorage->StoreFileData(inBuf.rptr(), nToStore))
 			{
+				dbgline;
 				sErrorMsg = "502 Could not store data";
 				return HINT_DISCON | EFLAG_JOB_BROKEN;
 			}
@@ -352,6 +363,7 @@ struct tDlJob
 				inBuf.drop(l);
 				if (h.type != header::ANSWER)
 				{
+					dbgline;
 					sErrorMsg = "500 Unexpected response type";
 					// smells fatal...
 					return EFLAG_MIRROR_BROKEN | HINT_DISCON;
@@ -428,13 +440,13 @@ struct tDlJob
 				}
 
 				// ok, can pass the data to the file handler
-				h.set(header::XORIG, RemoteUri());
-				fileitem_base::SuccessWithTransErrorFlag headStored = m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr(), bHotItem );
-				if(!headStored && headStored.xdata)
-					return EFLAG_LOST_CON | HINT_DISCON; // recoverable
-
-				if(!headStored)
+				h.set(header::XORIG, RemoteUri(false));
+				bool bDoRetry(false);
+				if(!m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr(), bHotItem, bDoRetry))
 				{
+					if(bDoRetry)
+						return EFLAG_LOST_CON | HINT_DISCON; // recoverable
+
 					ldbg("Item dl'ed by others or in error state --> drop it, reconnect");
 					m_DlState = STATE_PROCESS_DATA;
 					sErrorMsg = "502 Cache descriptor busy";
@@ -515,6 +527,14 @@ struct tDlJob
 		return EFLAG_JOB_BROKEN;
 	}
 
+	inline bool IsRecoverableState()
+	{
+		return (m_DlState == STATE_GETHEADER || m_DlState == STATE_REGETHEADER);
+		// XXX: In theory, could also easily recover from STATE_FINISH but that's
+		// unlikely to happen
+
+	}
+
 private:
 	// not to be copied ever
 	tDlJob(const tDlJob&);
@@ -586,7 +606,7 @@ inline void dlcon::EnqJob(tDlJob *todo)
 	ASSERT(todo->m_pStorage->m_nRangeLimit <0
 			|| todo->m_pStorage->m_nRangeLimit >= todo->m_pStorage->m_nSizeSeen);
 
-	LOGSTART2("dlcon::EnqJob", todo->m_fileUri.ToURI());
+	LOGSTART2("dlcon::EnqJob", todo->m_fileUri.ToURI(false));
 
 	m_qNewjobs.push_back(tDlJobPtr(todo));
 	if (m_wakepipe[1]>=0)
@@ -625,10 +645,10 @@ dlcon::~dlcon()
 	checkforceclose(m_wakepipe[1]);
 }
 
-inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueue &qActive)
+inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueue &inpipe)
 {
 	LOGSTART2("dlcon::ExchangeData",
-			"qsize: " << qActive.size() << ", sendbuf size: "
+			"qsize: " << inpipe.size() << ", sendbuf size: "
 			<< m_sendBuf.size() << ", inbuf size: " << m_inBuf.size());
 
 	fd_set rfds, wfds;
@@ -638,8 +658,8 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 	FD_ZERO(&rfds);
 	FD_ZERO(&wfds);
 
-	if(qActive.empty())
-		m_inBuf.clear(); // dirty buffer from previous connection?
+	if(inpipe.empty())
+		m_inBuf.clear(); // better be sure about dirty buffer from previous connection
 
 	// no socket operation needed in this case but just process old buffer contents
 	bool bReEntered=!m_inBuf.empty();
@@ -695,7 +715,13 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			dbgline;
 			sErrorMsg = "500 Connection timeout";
 			// was there anything to do at all?
-			return qActive.empty() ? HINT_SWITCH : (HINT_DISCON|EFLAG_JOB_BROKEN);
+			if(inpipe.empty())
+				return HINT_SWITCH;
+
+			if(inpipe.front()->IsRecoverableState())
+				return EFLAG_LOST_CON;
+			else
+				return (HINT_DISCON|EFLAG_JOB_BROKEN);
 		}
 
 		if (FD_ISSET(m_wakepipe[0], &rfds))
@@ -712,7 +738,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 
 			ldbg("Sending data...\n" << m_sendBuf);
 			int s = ::send(fd, m_sendBuf.data(), m_sendBuf.length(), MSG_NOSIGNAL);
-			ldbg("Sent " << s << " bytes from " << m_sendBuf.length());
+			ldbg("Sent " << s << " bytes from " << m_sendBuf.length() << " to " << con.get() );
 			if (s < 0)
 			{
 				// EAGAIN is weird but let's retry later, otherwise reconnect
@@ -772,7 +798,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 
 			proc_data:
 
-			if(qActive.empty())
+			if(inpipe.empty())
 			{
 				ldbg("FIXME: unexpected data returned?");
 				sErrorMsg = "500 Unexpected data";
@@ -782,11 +808,16 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			while(!m_inBuf.empty())
 			{
 
-				ldbg("Processing job for " << qActive.front()->RemoteUri());
-				UINT res = qActive.front()->ProcessIncomming(m_inBuf, false);
+				ldbg("Processing job for " << inpipe.front()->RemoteUri(false));
+				UINT res = inpipe.front()->ProcessIncomming(m_inBuf, false);
 				ldbg(
 						"... incoming data processing result: " << res
-						<< ", emsg: " << qActive.front()->sErrorMsg);
+						<< ", emsg: " << inpipe.front()->sErrorMsg);
+
+				if(res&EFLAG_MIRROR_BROKEN)
+				{
+					ldbg("###### BROKEN MIRROR ####### on " << con.get());
+				}
 
 				if (HINT_MORE == res)
 					goto loop_again;
@@ -795,17 +826,17 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 				{
 
 					// just in case that server damaged the last response body
-					con->KnowLastFile(WEAK_PTR<fileitem_base>(qActive.front()->m_pStorage));
+					con->KnowLastFile(WEAK_PTR<fileitem>(inpipe.front()->m_pStorage));
 
-					qActive.pop_front();
+					inpipe.pop_front();
 					if (HINT_DISCON & res)
 						return HINT_DISCON; // with cleaned flags
 
 					LOG(
-							"job finished. Has more? " << qActive.size()
+							"job finished. Has more? " << inpipe.size()
 							<< ", remaining data? " << m_inBuf.size());
 
-					if (qActive.empty())
+					if (inpipe.empty())
 					{
 						LOG("Need more work");
 						return HINT_SWITCH;
@@ -820,13 +851,13 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 					/* target was rewritten by the job handler, but there might be more redirections
 				    in the pipe which we can analyze and use ASAP to avoid later requests.
 				    Process as much as we can (carefully). */
-					tDljQueue::iterator it = qActive.begin();
-					for(++it; it != qActive.end(); ++it)
+					tDljQueue::iterator it = inpipe.begin();
+					for(++it; it != inpipe.end(); ++it)
 					{
 						UINT rr = (**it).ProcessIncomming(m_inBuf, true);
 						if( HINT_TGTCHANGE != rr )
 						{
-							m_nDisablePling=30;
+							m_nTempPipelineDisable=30;
 							return (HINT_TGTCHANGE|HINT_DISCON);
 						}
 					}
@@ -836,7 +867,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 				// else case: error handling, pass to main loop
 				if(HINT_KILL_LAST_FILE & res)
 					con->KillLastFile();
-				setIfNotEmpty(sErrorMsg, qActive.front()->sErrorMsg);
+				setIfNotEmpty(sErrorMsg, inpipe.front()->sErrorMsg);
 				return res;
 			}
 			return HINT_DONE; // input buffer consumed
@@ -853,7 +884,7 @@ inline void CleanRunning(tDljQueue &inpipe)
 	for(tDljQueue::iterator it = inpipe.begin(); it!= inpipe.end();)
 	{
 		if(*it && (**it).m_pStorage
-				&& (**it).m_pStorage->GetStatus() >= FIST_DLRECEIVING)
+				&& (**it).m_pStorage->GetStatus() >= fileitem::FIST_DLRECEIVING)
 		{
 			// someone else is doing it -> drop
 			inpipe.erase(it++);
@@ -888,32 +919,33 @@ void dlcon::WorkLoop()
 	bool bStopRequesting=false; // hint to stop adding request headers until the connection is restarted
 
 	int nLostConTolerance=0;
-#define MAX_RETRY_USED 8
-#define MAX_RETRY_FRESH 5
+#define MAX_RETRY 5
 
 	while(true) // outer loop: jobs, connection handling
 	{
+		// helper var, remember that the connection was recycled before
+		bool bUsed=false;
 
         // init state or transfer loop jumped out, what are the needed actions?
         {
         	setLockGuard;
         	LOG("New jobs: " << m_qNewjobs.size());
+
         	if(m_bStopASAP)
         	{
-        		// ordered to stop but someone might rely on the running job. Keep it running,
-        		// if the last user disappears and will cause the termination RSN
-        		if(inpipe.size()>1)
-        			inpipe.erase(++(inpipe.begin()), inpipe.end());
-
+        		/* The no-more-users checking logic will purge orphaned items from the inpipe
+        		 * queue. When the connection is dirty after that, it will be closed in the
+        		 * ExchangeData() but if not then it can be assumed to be clean and reusable.
+        		 */
         		if(inpipe.empty())
         		{
         			if(con)
         				tcpconnect::RecycleIdleConnection(con);
-
         			return;
         		}
         	}
 
+
         	if(m_qNewjobs.empty())
         		goto go_select; // parent will notify RSN
 
@@ -923,7 +955,6 @@ void dlcon::WorkLoop()
         		m_sendBuf.clear();
         		m_inBuf.clear();
 
-        		bool bUsed=false;
         		bStopRequesting=false;
 
         		for(tDljQueue::iterator it=m_qNewjobs.begin(); it!=m_qNewjobs.end();)
@@ -951,7 +982,8 @@ void dlcon::WorkLoop()
         					acfg::proxy_info.sPort, sErrorMsg, &bUsed,
         					m_qNewjobs.front()->GetConnStateTracker());
 
-        		nLostConTolerance = bUsed ? MAX_RETRY_USED : MAX_RETRY_FRESH;
+        		nLostConTolerance = MAX_RETRY + bUsed;
+
         		ldbg("connection valid? " << bool(con) << " was fresh? " << !bUsed);
 
         		if(con)
@@ -992,6 +1024,11 @@ void dlcon::WorkLoop()
         			bStopRequesting=true;
         			break;
         		}
+        		else if(inpipe.size() > UINT(acfg::pipelinelen))
+        		{
+        			LOG("Pipeline overrun, stop requesting for now");
+        			break;
+        		}
         		else
         		{
         			cjob->AppendRequest(m_sendBuf, m_sXForwardedFor);
@@ -999,15 +1036,18 @@ void dlcon::WorkLoop()
         			inpipe.push_back(cjob);
         			m_qNewjobs.pop_front();
 
-        			if(m_nDisablePling >0)
+        			if(m_nTempPipelineDisable >0)
         			{
         				bStopRequesting=true;
-        				--m_nDisablePling;
+        				--m_nTempPipelineDisable;
+        				break;
         			}
         		}
         	}
         }
 
+		ldbg("Request(s) cooked, buffer contents: " << m_sendBuf);
+
         go_select:
 
         if(inpipe.empty() && m_bManualMode)
@@ -1018,7 +1058,15 @@ void dlcon::WorkLoop()
         // either have something to receive or something to send (sooner or later)
 		ASSERT( !inpipe.empty() || !bStopRequesting);
 
-         // inner loop: plain communication until something happens. Maybe should use epoll here?
+		// must test this connection, just be sure no crap is in the pipe
+		if (bUsed && (bUsed = false, check_read_state(con->GetFD())))
+		{
+			ldbg("code: MoonWalker");
+			con.reset();
+			continue;
+		}
+
+        // inner loop: plain communication until something happens. Maybe should use epoll here?
         loopRes=ExchangeData(sErrorMsg, con, inpipe);
         ldbg("loopRes: "<< loopRes);
 
@@ -1054,7 +1102,7 @@ void dlcon::WorkLoop()
         	goto move_jobs_back_to_q;
         }
 
-		if ((EFLAG_LOST_CON & loopRes) && !inpipe.empty())
+        if ((EFLAG_LOST_CON & loopRes) && !inpipe.empty())
 		{
 			// disconnected by OS... give it a chance, or maybe not...
 			if (--nLostConTolerance <= 0)
@@ -1073,7 +1121,7 @@ void dlcon::WorkLoop()
 
         if(loopRes & (HINT_DONE|HINT_MORE))
         {
-        	nLostConTolerance=MAX_RETRY_FRESH;
+        	nLostConTolerance=MAX_RETRY;
         	sErrorMsg.clear();
         	continue;
         }
