@@ -23,9 +23,6 @@ using namespace MYSTD;
 #warning Unlinking parts defused
 #endif
 
-#define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
-#define SABSPATH(x) (CACHE_BASE+(x))
-
 #define ERRMSGABORT if(m_nErrorCount && m_bErrAbort) { SendChunk(sErr); return; }
 #define ERRABORT if(m_nErrorCount && m_bErrAbort) { return; }
 
@@ -34,6 +31,11 @@ using namespace MYSTD;
 
 #define FNAME_PENDING "_expending_dat"
 #define FNAME_DAMAGED "_expending_damaged"
+#define sFAIL_INI SABSPATH("_exfail_cnt")
+#define FAIL_INI sFAIL_INI.c_str()
+
+static cmstring sIndex("Index");
+static cmstring sslIndex("/Index");
 
 void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsumming)
 {
@@ -53,12 +55,13 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 			/* erases inside -> step forward there */ )
 	{
 		const tFileNdir &k = it->first; // shortcut
+		tDiskFileInfo &desc = it->second;
 
 		// where are we, still at the right filename? or could use range instead of lower_bound...
 		if(entry.sFileName != k.file)
 			return;
 
-		tFingerprint & fprHave=it->second.fpr;
+		tFingerprint & fprHave=desc.fpr;
 		const tFingerprint & fprNeed=entry.fpr;
 
 		string sPathRel(k.dirRel+k.file);
@@ -66,21 +69,24 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 		header h;
 		tS2IDX::const_iterator j;
 
-		if(ContHas(m_forcedTrashMap,sPathRel))
+		if(ContHas(m_forceKeepInTrash,sPathRel))
 			goto keep_in_trash;
 
-		// needs to match the exact file location
-		if(m_bNeedsStrictPathsHere)
+		// needs to match the exact file location.
+		// And for "Index" files, they have always to be at a well defined location, this
+		// constraint is also needed to expire deprecated files
+		if(m_bNeedsStrictPathsHere || entry.sFileName == sIndex)
 		{
+			// compare full paths (physical vs. remote) with their real paths
 			string sEntrPath=entry.sDirectory+entry.sFileName;
-			LOG("Checking exact path: " << sEntrPath << " vs. " << sPathAbs);
+			LOG("Checking exact path: " << sEntrPath << " vs. " << sPathRel);
 			pathTidy(sEntrPath);
 			if(sPathRel != sEntrPath)
-				goto keep_in_trash;
+				goto keep_in_trash; // not for us, try another one
 		}
 
 		// Basic header checks. Skip if the file was forcibly updated/reconstructed before.
-		if (m_bSkipHeaderChecks || it->second.bHeaderTestDone
+		if (m_bSkipHeaderChecks || desc.bHeaderTestDone
 				/*
 				|| (j = m_indexFilesRel.find(sPathRel)) == m_indexFilesRel.end()
 				|| j->second.uptodate
@@ -92,7 +98,7 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 		else
 		{
 			LOG("Doing basic header checks");
-			it->second.bHeaderTestDone = true;
+			desc.bHeaderTestDone = true;
 
 			if (0<h.LoadFromFile(sPathAbs+".head"))
 			{
@@ -109,21 +115,21 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 				off_t len=atoofft(h.h[header::CONTENT_LENGTH], -2);
 				if(len<0)
 				{
-					SendFmt() << "<font color=red>WARNING, header file of "
-							<< sPathRel << " does not contain content length</font>";
+					SendFmt << "<span class=\"ERROR\">WARNING, header file of "
+							<< sPathRel << " does not contain content length</span>";
 					goto keep_in_trash;
 				}
 				off_t lenInfo=GetFileSize(sPathAbs, -3);
 				if(lenInfo<0)
 				{
-					SendFmt() << ": error reading attributes, ignoring" << sPathRel;
+					SendFmt << ": error reading attributes, ignoring" << sPathRel;
 					goto keep_in_trash;
 				}
 				if (len < lenInfo)
 				{
-					SendFmt() << "<font color=red>WARNING, header file of " << sPathRel
+					SendFmt << "<span class=\"ERROR\">WARNING, header file of " << sPathRel
 							<< " reported too small file size (" << len << " vs. " << lenInfo
-							<< "), invalidating and removing header</font><br>\n";
+							<< "), invalidating and removing header</span><br>\n";
 					// kill the header ASAP, no matter which extra exception might apply later
 					ignore_value(::unlink((sPathAbs+".head").c_str()));
 					goto keep_in_trash;
@@ -134,8 +140,8 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 				tIfileAttribs at = GetFlags(sPathRel);
 				if(!at.parseignore && !at.forgiveDlErrors)
 				{
-					SendFmt()<<"<font color=\"orange\">WARNING, header file missing or damaged for "
-						<<sPathRel<<"</font><br>\n";
+					SendFmt<<"<span class=\"WARNING\">WARNING, header file missing or damaged for "
+						<<sPathRel<<"</span>\n<br>\n";
 				}
 			}
 		}
@@ -169,13 +175,13 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 				if (fprHave.size < entry.fpr.size && !m_bIncompleteIsDamaged)
 				{
 					l_tell_incomplete:
-					SendFmt() << "<font color=orange>INCOMPLETE: " << sPathRel
-							<< " (ignoring)</font><br>\n";
+					SendFmt << "<span class=\"WARNING\">INCOMPLETE: " << sPathRel
+							<< " (ignoring)</span><br>\n";
 				}
 				else if (m_bTruncateDamaged)
 				{
-					SendFmt() << "<font color=orange>BAD: " << sPathRel
-							<< " (truncating)</font><br>\n";
+					SendFmt << "<span class=\"WARNING\">BAD: " << sPathRel
+							<< " (truncating)</span><br>\n";
 					ignore_value(::truncate(sPathAbs.c_str(), 0));
 				}
 				else
@@ -184,15 +190,16 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 
 					if(GetFlags(sPathRel).parseignore)
 					{
-						SendFmt() << sPathRel << ": incorrect or obsolete contents but marked as "
-								"alternative version of another index file <font color=orange>(ignoring)</font>";
+						SendFmt << sPathRel << ": incorrect or obsolete contents but marked as "
+								"alternative version of another index file "
+								"<span class=\"WARNING\">(ignoring)</span>";
 					}
 					else
 					{
 						if (m_damageList.is_open())
 							m_damageList << sPathRel << endl;
-						SendFmt() << "<font color=red>BAD: " << sPathRel
-								<< " (adding to damage list)</font>";
+						SendFmt << "<span class=\"ERROR\">BAD: " << sPathRel
+								<< " (adding to damage list)</span>";
 						AddDelCbox(sPathRel);
 					}
 					SendChunk("<br>\n");
@@ -204,7 +211,7 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 
 		// ok, package matched, contents ok if checked, drop it from the removal list
 		if (m_bVerbose)
-			SendFmt() << "<font color=green>OK: " << sPathRel << "</font><br>\n";
+			SendFmt << "<font color=green>OK: " << sPathRel << "</font><br>\n";
 
 
 		{ // goto side effects :-(
@@ -262,6 +269,8 @@ struct tVerCompKey
 	}
 };
 
+// this method looks for the validity of additional package files kept in cache after
+// the Debian version moved to a higher one
 inline void expiration::DropExceptionalVersions()
 {
     if(m_trashCandSet.empty() || !acfg::keepnver)
@@ -309,7 +318,7 @@ inline void expiration::DropExceptionalVersions()
 		pkgGroup.insert(key);
 	}
 	if(ec)
-		SendFmt() << "<br><b>Kept "
+		SendFmt << "<br><b>Kept "
 		<< ec << " more package(s) not covered by index references</b><br><br>\n";
 }
 
@@ -322,19 +331,19 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 
         DropExceptionalVersions();
 
-        string sDbFileAbs=CACHE_BASE+"_expending_dat";
+        string sDbFileAbs=CACHE_BASE+FNAME_PENDING;
 
         f = fopen(sDbFileAbs.c_str(), "w");
         if(!f)
         {
-            SendChunk("Unable to open _expending.dat for writing, trying to recreate... ");
+            SendChunk("Unable to open " FNAME_PENDING " for writing, attempting to recreate... ");
             ::unlink(sDbFileAbs.c_str());
             f=::fopen(sDbFileAbs.c_str(), "w");
             if(f)
-                SendChunk("OK<br>\n");
+                SendChunk("OK\n<br>\n");
             else
             {
-                SendChunk("<font color=red>FAILED. ABORTING. Check filesystem and file permissions.");
+                SendChunk("<span class=\"ERROR\">FAILED. ABORTING. Check filesystem and file permissions.</span>");
                 return;
             }
         }
@@ -347,16 +356,17 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 		using namespace rechecks;
 		const tFileNdir &k = it->first; // shortcut
 		string sPathRel = k.dirRel+k.file;
+		DBGQLOG("Checking " << sPathRel);
 
-		if(ContHas(m_forcedTrashMap,sPathRel))
+		if(ContHas(m_forceKeepInTrash,sPathRel))
 		{
-			LOG("forcetrash flag set, whitelist does not apply, consider removing");
+			LOG("forcetrash flag set, whitelist does not apply, not to be removed");
 		}
 		else
 		{
 			if (Match(k.file, WHITELIST) || Match(sPathRel, WHITELIST))
 			{
-				LOG("File not to be removed, ignoring");
+				LOG("Protected file, not to be removed");
 				continue;
 			}
 		}
@@ -376,7 +386,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 
 		if(bPurgeNow || TIMEEXPIRED(it->second.nLostAt))
 		{
-			SendFmt() << "Removing " << sPathRel << "<br>\n";
+			SendFmt << "Removing " << sPathRel << "<br>\n";
 
 #ifdef ENABLED
 			::unlink(sPathAbs.c_str());
@@ -386,9 +396,9 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
         }
 		else if(f)
 		{
-			SendFmt() << "Tagging " << sPathRel;
+			SendFmt << "Tagging " << sPathRel;
 			if(m_bVerbose)
-				SendFmt() << " (t-"<<(m_gMaintTimeNow-it->second.nLostAt)/3600 << "h)";
+				SendFmt << " (t-"<<(m_gMaintTimeNow-it->second.nLostAt)/3600 << "h)";
 			SendChunk("<br>\n");
 
 			n++;
@@ -413,7 +423,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 		}
 
 		if(m_bVerbose)
-			SendFmt() << "Removing orphaned head file: " << sPathRel;
+			SendFmt << "Removing orphaned head file: " << sPathRel;
 
 #ifdef ENABLED
         string sPathAbs=CACHE_BASE+sPathRel;
@@ -427,17 +437,18 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
     	TellCount(n, tagSpace);
 }
 
+
 void expiration::Action(const string & cmd)
 {
 	if (m_mode==purge)
 	{
-		LoadTrashMapFromFile(true);
+		LoadPreviousData(true);
 		RemoveAndStoreStatus(true);
 		return;
 	}
 	if (m_mode==list)
 	{
-		LoadTrashMapFromFile(true);
+		LoadPreviousData(true);
 		off_t nSpace(0);
 		uint cnt(0);
 		for (tS2DAT::iterator it=m_trashCandSet.begin(); it
@@ -464,7 +475,7 @@ void expiration::Action(const string & cmd)
 
 		mstring delURL(cmd);
 		StrSubst(delURL, "justShow", "justRemove");
-		SendFmt() << "<a href=\"/"<<delURL<<"\">Delete all listed files</a> "
+		SendFmtRemote << "<a href=\"/"<<delURL<<"\">Delete all listed files</a> "
 				"(no further confirmation)<br>\n";
 		return;
 	}
@@ -485,17 +496,17 @@ void expiration::Action(const string & cmd)
 
 			if(m_mode == purgeDamaged)
 			{
-				SendFmt() << "Removing " << s << "<br>\n";
+				SendFmt << "Removing " << s << "<br>\n";
 				::unlink(SZABSPATH(s));
 				::unlink(SZABSPATH(s+".head"));
 			}
 			else if(m_mode == truncDamaged)
 			{
-				SendFmt() << "Truncating " << s << "<br>\n";
+				SendFmt << "Truncating " << s << "<br>\n";
 				ignore_value(::truncate(SZABSPATH(s), 0));
 			}
 			else
-				SendFmt() << s << "<br>\n";
+				SendFmt << s << "<br>\n";
 		}
 		return;
 	}
@@ -507,63 +518,78 @@ void expiration::Action(const string & cmd)
 	SendChunk("<b>Locating potentially expired files in the cache...</b><br>\n");
 
 	DirectoryWalk(acfg::cachedir, this);
-	if(CheckAbortCondition())
-		return;
-	SendFmt()<<"Found "<<m_nProgIdx<<" files.<br />\n";
+	if(CheckStopSignal())
+		goto save_fail_count;
+	SendFmt<<"Found "<<m_nProgIdx<<" files.<br />\n";
 
 	//cout << "found package files: " << m_trashCandidates.size()<<endl;
 	//for(tS2DAT::iterator it=m_trashCandSet.begin(); it!=m_trashCandSet.end(); it++)
 	//	SendChunk(tSS()<<it->second.sDirname << "~~~" << it->first << " : " << it->second.fpr.size<<"<br>");
 
-	LoadIgnoreList();
+	LoadHints();
 	UpdateIndexFiles();
 
-	if(CheckAbortCondition())
-		return;
-
-	if (m_nErrorCount > 0 && m_bErrAbort)
-	{
-		SendChunk("<font color=\"red\">Found errors during processing, "
-				"aborting as requested.</font><!-- TELL:THE:ADMIN -->");
-		return;
-	}
+	if(CheckAndReportError() || CheckStopSignal())
+		goto save_fail_count;
 
 	m_damageList.open(SZABSPATH(FNAME_DAMAGED), ios::out | ios::trunc);
 
 	SendChunk("<b>Validating cache contents...</b><br>\n");
 	_ProcessSeenIndexFiles();
 
-	if (m_bErrAbort && m_nErrorCount>0)
-	{
-		SendChunk("<font color=\"red\">Found errors during processing, "
-				"aborting as requested.</font>"
-			"<!-- TELL:THE:ADMIN -->");
-		return;
-	}
-
-	if(CheckAbortCondition())
-			return;
+	if(CheckAndReportError() || CheckStopSignal())
+		goto save_fail_count;
 
 	// update timestamps of pending removals
-	LoadTrashMapFromFile(false);
+	LoadPreviousData(false);
 
 	RemoveAndStoreStatus(cmd.find("purgeNow")!=stmiss);
 	PurgeMaintLogs();
 
 	DelTree(CACHE_BASE+"_actmp");
 
-	PrintStats("Assigned disk space");
+	PrintStats("Allocated disk space");
 
 	SendChunk("<br>Done.</br>");
 
+	save_fail_count:
+
+	if (m_nErrorCount <= 0)
+	{
+		::unlink(FAIL_INI);
+	}
+	else
+	{
+		FILE *f = fopen(FAIL_INI, "a");
+		if (!f)
+		{
+			SendFmt << "Unable to open " <<
+			sFAIL_INI << " for writing, attempting to recreate... ";
+			::unlink(FAIL_INI);
+			f = ::fopen(FAIL_INI, "w");
+			if (f)
+				SendChunk("OK\n<br>\n");
+			else
+			{
+				SendChunk(
+						"<span class=\"ERROR\">FAILED. ABORTING. Check filesystem and file permissions.</span>");
+			}
+		}
+		if (f)
+		{
+			::fprintf(f, "%lu\n", (long unsigned int) GetTime());
+			checkForceFclose(f);
+		}
+	}
+
 }
 
 void expiration::PurgeMaintLogs()
 {
-	tStrDeq logs = ExpandFilePattern(acfg::logdir + SZPATHSEP"maint_*.log*");
+	tStrDeq logs = ExpandFilePattern(acfg::logdir + SZPATHSEP MAINT_PFX "*.log*");
 	if (logs.size() > 2)
 		SendChunk(
-				"Found required cleanup tasks: purging maintanence logs...<br>\n");
+				"Found required cleanup tasks: purging maintenance logs...<br>\n");
 	for (tStrDeq::const_iterator it = logs.begin(); it != logs.end(); it++)
 	{
 		time_t id = atoofft(it->c_str() + acfg::logdir.size() + 7);
@@ -605,10 +631,9 @@ void expiration::UpdateFingerprint(const MYSTD::string &sPathRel,
 		*/
 	else // just update the file size
 	{
-		//SendChunk(tSS() << "cs from fresh stat info<br>");
-		struct stat stbuf;
 		finfo.fpr.csType=CSTYPE_INVALID;
-		if (::stat((CACHE_BASE+sPathRel).c_str(), &stbuf))
+		Cstat stbuf(CACHE_BASE+sPathRel);
+		if (!stbuf)
 			aclog::err(sPathRel + " << FAILED TO READ");
 		else
 			finfo.fpr.Set(NULL, CSTYPE_INVALID, stbuf.st_size);
@@ -619,7 +644,7 @@ void expiration::UpdateFingerprint(const MYSTD::string &sPathRel,
 bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stinfo)
 {
 
-	if(CheckAbortCondition())
+	if(CheckStopSignal())
 		return false;
 
 	if(sPathAbs.size()<=CACHE_BASE_LEN) // heh?
@@ -628,6 +653,7 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 	ProgTell();
 
 	string sPathRel(sPathAbs, CACHE_BASE_LEN);
+	DBGQLOG(sPathRel);
 
 	// detect strings which are only useful for shell or js attacks
 	if(sPathRel.find_first_of("\r\n'\"<>")!=stmiss)
@@ -641,7 +667,17 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 		m_trashCandHeadSet.insert(sPathRel);
 	else
 	{
-		AddIFileCandidate(sPathRel);
+		if(AddIFileCandidate(sPathRel))
+		{
+			tIfileAttribs &attr = SetFlags(sPathRel);
+			attr.space+=stinfo.st_size;
+			if(endsWith(sPathRel, sslIndex))
+			{
+				attr.forgiveDlErrors = true;
+				//attr.hideDlErrors = true;
+			}
+		}
+
 //#warning TODO: when Lenny is history, add additional MD5SUMS file to the list for d-i image files
 		//SendChunk(string("<br>hm?<br>")+sDirnameAndSlash + " -- " + sBasename);
 		tDiskFileInfo &finfo = m_trashCandSet[sPathRel];
@@ -652,7 +688,8 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 }
 
 
-expiration::expiration(int fd, workType type) : tCacheMan(fd), m_mode(type), m_bIncompleteIsDamaged(false)
+expiration::expiration(int fd, workType type) :
+		tCacheMan(fd), m_mode(type), m_bIncompleteIsDamaged(false), m_nPrevFailCount(0)
 {
 	switch(type)
 	{
@@ -670,7 +707,7 @@ expiration::~expiration()
 {
 }
 
-void expiration::LoadIgnoreList()
+void expiration::LoadHints()
 {
 	filereader reader;
 	if(!reader.OpenFile(acfg::confdir+SZPATHSEP+"ignore_list"))
@@ -690,8 +727,16 @@ void expiration::LoadIgnoreList()
 			continue;
 		SetFlags(sTmp).forgiveDlErrors=true;
 	}
+
+	reader.Close();
+	reader.OpenFile(sFAIL_INI);
+	while(reader.GetOneLine(sTmp))
+		m_nPrevFailCount += (atol(sTmp.c_str())>0);
+
+
+
 }
-void expiration::LoadTrashMapFromFile(bool bForceInsert)
+void expiration::LoadPreviousData(bool bForceInsert)
 {
 	filereader reader;
 	reader.OpenFile(SABSPATH(FNAME_PENDING));
@@ -770,5 +815,17 @@ void expiration::LoadTrashMapFromFile(bool bForceInsert)
 
 }
 
-
+inline bool expiration::CheckAndReportError()
+{
+	if (m_nErrorCount > 0 && m_bErrAbort)
+	{
+		SendChunk("<span class=\"ERROR\">Found errors during processing, "
+				"aborting as requested.</span>");
+		SendChunk(m_nPrevFailCount+(m_nErrorCount>0) > acfg::exsupcount
+				? "<!-- TELL:THE:ADMIN -->"
+						: "<!-- NOT:TELLING:THE:ADMIN:YET -->");
+		return true;
+	}
+	return false;
+}
 
