@@ -18,12 +18,15 @@ using namespace MYSTD;
 #define MAXTEMPDELAY acfg::maxtempdelay // 27
 mstring sReplDir("_altStore" SZPATHSEP);
 
-header const & fileitem_base::GetHeaderUnlocked()
+static tFiGlobMap mapItems;
+lockable mapLck;
+
+header const & fileitem::GetHeaderUnlocked()
 {
 	return m_head;
 }
 
-string fileitem_base::GetHttpMsg()
+string fileitem::GetHttpMsg()
 {
 	setLockGuard;
 	if(m_head.frontLine.length()>9)
@@ -31,8 +34,7 @@ string fileitem_base::GetHttpMsg()
 	return m_head.frontLine;
 }
 
-time_t fileitem_base::m_nEarliestExpiration(END_OF_TIME);
-
+/*
 fileitem_base::fileitem_base(MYSTD::string sPath) :
 	condition(),
 	m_nIncommingCount(0),
@@ -45,95 +47,49 @@ fileitem_base::fileitem_base(MYSTD::string sPath) :
 	m_filefd(-1),
 	m_nDlRefsCount(0),
 	m_status(FIST_FRESH),
-	m_nTimeExpireAt(0),
 	m_nTimeDlDone(END_OF_TIME)
 {
 
-	// FIXME: this sucks, belongs into the fileitem
-	if(acfg::stupidfs)
-	{
-		// fix weird chars that we don't like in the filesystem
-		replaceChars(sPath, ENEMIESOFDOSFS, '_');
-		replaceChars(sPath, ENEMIESOFDOSFS, '_');
-#ifdef WIN32
-		replaceChars(sPath, "/", '\\');
-#endif
-	}
-
-	// not using hexmap yet, convert just what is really needed for APT
-
-	for (int i = 0; i < int(sPath.length()) - 3; i++)
-	{
-		if (sPath[i] == '%')
-		{
-			switch (sPath[i + 1])
-			{
-			case '7':
-				switch (sPath[i + 2])
-				{
-				case 'E':
-				case 'e':
-					sPath.replace(i, 3, "~", 1);
-					break;
-				}
-				break;
-			case '5':
-				switch (sPath[i + 2])
-				{
-				case 'B':
-				case 'b':
-					sPath.replace(i, 3, "[", 1);
-					break;
-				case 'D':
-				case 'd':
-					sPath.replace(i, 3, "]", 1);
-					break;
-				}
-				break;
-			default:
-				continue;
-			}
-		}
-	}
-
 	m_sPathRel = sPath;
 	m_sPathAbs = acfg::cacheDirSlash+sPath;
 
 	//ldbg("item created, m_sPath: "<< m_sPath);
 }
+*/
 
-
-fileitem_base::fileitem_base() :
+fileitem::fileitem() :
 	condition(),
 	m_nIncommingCount(0),
 	m_nSizeSeen(0),
+	m_nRangeLimit(-1),
 	m_bCheckFreshness(true),
 	m_bHeadOnly(false),
-	m_nRangeLimit(-1),
 	m_bAllowStoreData(true),
 	m_nSizeChecked(0),
 	m_filefd(-1),
 	m_nDlRefsCount(0),
 	m_status(FIST_FRESH),
-	m_nTimeExpireAt(0),
-	m_nTimeDlDone(END_OF_TIME)
+	m_nTimeDlStarted(0),
+	m_nTimeDlDone(END_OF_TIME),
+	m_globRef(mapItems.end()),
+	usercount(0)
 {
 }
 
-fileitem_base::~fileitem_base()
+fileitem::~fileitem()
 {
 	//setLockGuard;
 //	m_head.clear();
 	checkforceclose(m_filefd);
 }
 
-void fileitem_base::IncDlRefCount()
+void fileitem::IncDlRefCount()
 {
 	setLockGuard;
 	m_nDlRefsCount++;
 }
 
-void fileitem_base::DecDlRefCount(const string &sReason)
+void fileitem::DecDlRefCount(const string &sReason)
 {
 	setLockGuard;
 	
@@ -143,11 +99,11 @@ void fileitem_base::DecDlRefCount(const string &sReason)
 	if(m_nDlRefsCount>0)
 		return; // someone will care...
 	
-	// ... otherwise: the last downloader disappeared, need to tell observers
+	// ... otherwise: the last downloader disappeared, needing to tell observers
 
 	if (m_status<FIST_COMPLETE)
 	{
-		m_status=FIST_LOCALERROR;
+		m_status=FIST_DLERROR;
 		m_head.clear();
 		m_head.frontLine=string("HTTP/1.1 ")+sReason;
 		m_head.type=header::ANSWER;
@@ -158,7 +114,7 @@ void fileitem_base::DecDlRefCount(const string &sReason)
 	checkforceclose(m_filefd);
 }
 
-uint64_t fileitem_base::GetTransferCount()
+uint64_t fileitem::GetTransferCount()
 {
 	setLockGuard;
 	uint64_t ret=m_nIncommingCount;
@@ -166,12 +122,12 @@ uint64_t fileitem_base::GetTransferCount()
 	return ret;
 }
 
-int fileitem_base::GetFileFd() {
+int fileitem::GetFileFd() {
 	LOGSTART("fileitem::GetFileFd");
 	setLockGuard;
 
-	ldbg("Opening " << m_sPathAbs);
-	int fd=open(m_sPathAbs.c_str(), O_RDONLY);
+	ldbg("Opening " << m_sPathRel);
+	int fd=open(SZABSPATH(m_sPathRel), O_RDONLY);
 	
 #ifdef HAVE_FADVISE
 	// optional, experimental
@@ -189,7 +145,7 @@ off_t GetFileSize(cmstring & path, off_t defret)
 }
 
 
-void fileitem_base::ResetCacheState()
+void fileitem::ResetCacheState()
 {
 	setLockGuard;
 	m_nSizeSeen = 0;
@@ -199,7 +155,7 @@ void fileitem_base::ResetCacheState()
 	m_head.clear();
 }
 
-FiStatus fileitem_base::Setup(bool bCheckFreshness)
+fileitem::FiStatus fileitem::Setup(bool bCheckFreshness)
 {
 	LOGSTART2("fileitem::Setup", bCheckFreshness);
 
@@ -207,17 +163,21 @@ FiStatus fileitem_base::Setup(bool bCheckFreshness)
 
 	if(m_status>FIST_FRESH)
 		return m_status;
+
 	m_status=FIST_INITED;
 	m_bCheckFreshness = bCheckFreshness;
 	
-	if(m_head.LoadFromFile(m_sPathAbs+".head") >0 && m_head.type==header::ANSWER )
+	cmstring sPathAbs(CACHE_BASE+m_sPathRel);
+
+	if(m_head.LoadFromFile(sPathAbs+".head") >0 && m_head.type==header::ANSWER )
 	{
 		if(200 != m_head.getStatus())
 			goto error_clean;
 		
+
 		LOG("good head");
 
-		m_nSizeSeen=GetFileSize(m_sPathAbs, 0);
+		m_nSizeSeen=GetFileSize(sPathAbs, 0);
 
 		// some plausibility checks
 		if(m_bCheckFreshness)
@@ -267,20 +227,20 @@ FiStatus fileitem_base::Setup(bool bCheckFreshness)
 		// maybe there is some left-over without head file?
 		// Don't thrust volatile data, but otherwise try to reuse?
 		if(!bCheckFreshness)
-			m_nSizeSeen=GetFileSize(m_sPathAbs, 0);
+			m_nSizeSeen=GetFileSize(sPathAbs, 0);
 	}
 	LOG("resulting status: " << m_status);
 	return m_status;
 
 	error_clean:
-			unlink((m_sPathAbs+".head").c_str());
+			::unlink((sPathAbs+".head").c_str());
 			m_head.clear();
 			m_nSizeSeen=0;
 			m_status=FIST_INITED;
 			return m_status; // unuseable, to be redownloaded
 }
 
-bool fileitem_base::CheckUsableRange_unlocked(off_t nRangeLastByte)
+bool fileitem::CheckUsableRange_unlocked(off_t nRangeLastByte)
 {
 	if(m_status == FIST_COMPLETE)
 		return true;
@@ -295,14 +255,17 @@ bool fileitem_base::CheckUsableRange_unlocked(off_t nRangeLastByte)
 			&& atoofft(m_head.h[header::CONTENT_LENGTH], -255) > nRangeLastByte);
 }
 
-bool fileitem_base::SetupClean(bool bForce)
+bool fileitem::SetupClean(bool bForce)
 {
 	setLockGuard;
 
 	if(bForce)
 	{
 		if(m_status>FIST_FRESH)
-			m_status = FIST_LOCALERROR;
+		{
+			m_status = FIST_DLERROR;
+			m_head.frontLine="HTTP/1.1 500 FIXME, DEAD ITEM";
+		}
 	}
 	else
 	{
@@ -310,7 +273,8 @@ bool fileitem_base::SetupClean(bool bForce)
 			return false;
 		m_status=FIST_INITED;
 	}
-	if(::truncate(m_sPathAbs.c_str(), 0) || ::truncate((m_sPathAbs+".head").c_str(), 0))
+	cmstring sPathAbs(CACHE_BASE+m_sPathRel);
+	if(::truncate(sPathAbs.c_str(), 0) || ::truncate((sPathAbs+".head").c_str(), 0))
 		return false;
 
 	m_head.clear();
@@ -319,7 +283,7 @@ bool fileitem_base::SetupClean(bool bForce)
 	return true;
 }
 
-void fileitem_base::SetupComplete()
+void fileitem::SetupComplete()
 {
 	setLockGuard;
 	notifyAll();
@@ -327,10 +291,21 @@ void fileitem_base::SetupComplete()
 	m_status = FIST_COMPLETE;
 }
 
+fileitem::FiStatus fileitem::WaitForFinish(int *httpCode)
+{
+	setLockGuard;
+	while(m_status<FIST_COMPLETE)
+		wait();
+	if(httpCode)
+		*httpCode=m_head.getStatus();
+	return m_status;
+}
+
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
 	aclog::errnoFmter f;
-	aclog::err( (sFile+" storage error ["+msg+"], last errno: "+f).c_str());
+	aclog::err(tSS() << sFile <<
+			" storage error [" << msg << "], last errno: " << f);
 }
 
 #ifndef MINIBUILD
@@ -346,32 +321,31 @@ inline void _LogWithErrno(const char *msg, const string & sFile)
 #define SET_FRONTLINE(ret, x) ret="HTTP/1.1 "; ret+=x;
 #define SETERROR(x) { m_bAllowStoreData=false; SET_FRONTLINE(m_head.frontLine, x); \
 	m_head.set(header::XORIG, h.h[header::XORIG]); \
-    m_status=FIST_LOCALERROR; m_nTimeDlDone=GetTime(); _LogWithErrno(x, m_sPathRel); }
+    m_status=FIST_DLERROR; m_nTimeDlDone=GetTime(); _LogWithErrno(x, m_sPathRel); }
 #define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
 #define BOUNCE(x) { SETERROR(x); return false; }
 
-fileitem::SuccessWithTransErrorFlag
-fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *pNextData,
-		bool bForcedRestart)
+bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *pNextData,
+		bool bForcedRestart, bool &bDoCleanRetry)
 {
 	LOGSTART("fileitem::DownloadStartedStoreHeader");
 
 	setLockGuard;
 
-	USRDBG(5, "Download started, storeHeader for " << m_sPathRel << ", current status: " << m_status);
+	USRDBG( "Download started, storeHeader for " << m_sPathRel << ", current status: " << m_status);
 	
 	if(m_status >= FIST_COMPLETE)
 	{
-		USRDBG(0, "Error: item finished and idle, refusing download start");
+		USRDBG( "Download was completed or aborted, not restarting before expiration");
 		return false;
 	}
 
-	// conflict with another thread's initial attempt? Deny, except for a forced restart
+	// conflict with another thread's download attempt? Deny, except for a forced restart
 	if (m_status > FIST_DLPENDING && !bForcedRestart)
 		return false;
 	
 	if(m_bCheckFreshness)
-		m_nTimeExpireAt = GetTime() + MAXTEMPDELAY;
+		m_nTimeDlStarted = GetTime();
 
 	m_nIncommingCount+=h.m_nEstimLength;
 
@@ -381,7 +355,9 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 	// status will change, most likely... ie. BOUNCE action
 	notifyAll();
 
-	string sHeadPath=m_sPathAbs+".head";
+	cmstring sPathAbs(CACHE_BASE+m_sPathRel);
+	string sHeadPath=sPathAbs + ".head";
+
 	int serverStatus = h.getStatus();
 #if 0
 #warning FIXME
@@ -405,7 +381,7 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 				/* shouldn't be here, server should have resumed at the previous position.
 				 * Most likely the remote file was modified after the download started.
 				 */
-				//USRDBG(0, "state: " << m_status << ", m_nsc: " << m_nSizeChecked);
+				//USRDBG( "state: " << m_status << ", m_nsc: " << m_nSizeChecked);
 				BOUNCE("500 Failed to resume remote download");
 			}
 			if(h.h[header::CONTENT_LENGTH] && atoofft(h.h[header::CONTENT_LENGTH])
@@ -474,7 +450,7 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		// special optimization; if "-1 trick" was used then maybe don't reopen that file for writing later
 		if(m_bCheckFreshness && pNextData && m_nSizeSeen == mylen && m_nSizeChecked == mylen-1)
 		{
-			int fd=open(m_sPathAbs.c_str(), O_RDONLY);
+			int fd=open(sPathAbs.c_str(), O_RDONLY);
 			if(fd>=0)
 			{
 				if(m_nSizeChecked==lseek(fd, m_nSizeChecked, SEEK_SET))
@@ -482,8 +458,8 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 					char c;
 					if(1 == read(fd, &c, 1) && c == *pNextData)
 					{
-						aclog::err("known data hit, don't write to...");
-						aclog::err(m_sPathAbs);
+						if(acfg::debug & LOG_DEBUG)
+							aclog::err(tSS() << "known data hit, don't write to: "<< m_sPathRel);
 						m_bAllowStoreData=false;
 						m_nSizeChecked=mylen;
 					}
@@ -495,14 +471,15 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		break;
 	}
 	case 416:
-		// that's bad; it cannot have been complete before (the -1 trick)
+		// that's bad; it cannot have been completed before (the -1 trick)
 		// however, proxy servers with v3r4 cl3v3r caching strategy can cause that
 		// if if-mo-since is used and they don't like it, so attempt a retry in this case
 		if(m_nSizeChecked == 0)
 		{
-			USRDBG(0, "Peer denied to resume previous download (transient error) " << m_sPathRel );
+			USRDBG( "Peer denied to resume previous download (transient error) " << m_sPathRel );
 			m_nSizeSeen = 0;
-			return SuccessWithTransErrorFlag(false, true);
+			bDoCleanRetry=true;
+			return false;
 		}
 		else
 		{
@@ -520,7 +497,7 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		{
 			// got an error from the replacement mirror? cannot handle it properly
 			// because some job might already have started returning the data
-			USRDBG(0, "Cannot restart, HTTP code: " << serverStatus);
+			USRDBG( "Cannot restart, HTTP code: " << serverStatus);
 			BOUNCE(h.getCodeMessage());
 		}
 
@@ -548,8 +525,8 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		int flags = O_WRONLY | O_CREAT | O_BINARY;
 		struct stat stbuf;
 				
-		mkbasedir(m_sPathAbs);
-		m_filefd=open(m_sPathAbs.c_str(), flags, acfg::fileperms);
+		mkbasedir(sPathAbs);
+		m_filefd=open(sPathAbs.c_str(), flags, acfg::fileperms);
 		ldbg("file opened?! returned: " << m_filefd);
 		
 		// self-recovery from cache poisoned with files with wrong permissions
@@ -557,27 +534,27 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		{
 			if(m_nSizeChecked>0) // OOOH CRAP! CANNOT APPEND HERE! Do what's still possible.
 			{
-				string temp=m_sPathAbs+".tmp";
-				if(FileCopy(m_sPathAbs, temp) && 0==unlink(m_sPathAbs.c_str()) )
+				string temp=sPathAbs+".tmp";
+				if(FileCopy(sPathAbs, temp) && 0==unlink(sPathAbs.c_str()) )
 				{
-					if(0!=rename(temp.c_str(), m_sPathAbs.c_str()))
+					if(0!=rename(temp.c_str(), sPathAbs.c_str()))
 						BOUNCE("503 Cannot rename files");
 					
 					// be sure about that
-					if(0!=stat(m_sPathAbs.c_str(), &stbuf) || stbuf.st_size!=m_nSizeSeen)
+					if(0!=stat(sPathAbs.c_str(), &stbuf) || stbuf.st_size!=m_nSizeSeen)
 						BOUNCE("503 Cannot copy file parts, filesystem full?");
 					
-					m_filefd=open(m_sPathAbs.c_str(), flags, acfg::fileperms);
+					m_filefd=open(sPathAbs.c_str(), flags, acfg::fileperms);
 					ldbg("file opened after copying around: ");
 				}
 				else
 					BOUNCE((tSS()<<"503 Cannot store or remove files in "
-							<< GetDirPart(m_sPathAbs)).c_str());
+							<< GetDirPart(sPathAbs)).c_str());
 			}
 			else
 			{
-				unlink(m_sPathAbs.c_str());
-				m_filefd=open(m_sPathAbs.c_str(), flags, acfg::fileperms);
+				unlink(sPathAbs.c_str());
+				m_filefd=open(sPathAbs.c_str(), flags, acfg::fileperms);
 				ldbg("file force-opened?! returned: " << m_filefd);
 			}
 		}
@@ -586,7 +563,7 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		{
 			aclog::errnoFmter efmt("503 Cache storage error - ");
 #ifdef DEBUG
-			BOUNCE((efmt+m_sPathAbs).c_str());
+			BOUNCE((efmt+sPathAbs).c_str());
 #else
 			BOUNCE(efmt.c_str());
 #endif
@@ -639,29 +616,31 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		forceclose(m_filefd);
 	}
 
-	LOG("Deleting " << m_sPathAbs);
-	unlink(m_sPathAbs.c_str());
+	LOG("Deleting " << sPathAbs);
+	unlink(sPathAbs.c_str());
 	unlink(sHeadPath.c_str());
 	
-	m_status=FIST_LOCALERROR;
+	m_status=FIST_DLERROR;
 	m_nTimeDlDone=GetTime();
 	return false;
 }
 
 bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 {
-
 	setLockGuard;
 
 	LOGSTART2("fileitem::StoreFileData", "status: " << m_status << ", size: " << size);
 
 	// something might care, most likely... also about BOUNCE action
 	notifyAll();
-	
+
 	m_nIncommingCount+=size;
 	
-	if(m_status >= FIST_LOCALERROR || m_status < FIST_DLGOTHEAD)
+	if(m_status > FIST_COMPLETE || m_status < FIST_DLGOTHEAD)
+	{
+		ldbg("StoreFileData rejected, status: " << m_status)
 		return false;
+	}
 	
 	if (size==0)
 	{
@@ -681,7 +660,7 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 			if (m_filefd >= 0 && !m_head.h[header::CONTENT_LENGTH])
 			{
 				m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
-				m_head.StoreToFile(m_sPathAbs + ".head");
+				m_head.StoreToFile(CACHE_BASE + m_sPathRel + ".head");
 			}
 		}
 	}
@@ -700,7 +679,8 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 						continue;
 					aclog::errnoFmter efmt("HTTP/1.1 503 ");
 					m_head.frontLine = efmt;
-					m_status=FIST_LOCALERROR;
+					m_status=FIST_DLERROR;
+					// message will be set by the caller
 					m_nTimeDlDone=GetTime();
 					_LogWithErrno(efmt.c_str(), m_sPathRel);
 					return false;
@@ -719,58 +699,65 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 	return true;
 }
 
-//typedef MYSTD::pair<int,tFileItemPtr> tFileRefEntry;
-//typedef MYSTD::multimap<string, tFileRefEntry> tFiGlobMap;
-static tFiGlobMap mapItems;
-lockable mapLck;
-
-fileitem::fileitem(cmstring &s) : fileitem_with_storage(s), m_globRef(mapItems.end())
-{};
-
-void fileitem::Unreg()
+fileItemMgmt::~fileItemMgmt()
 {
-	LOGSTART("fileitem::Unreg");
+	LOGSTART("fileItemMgmt::~fileItemMgmt");
+	Unreg();
+}
+
+inline void fileItemMgmt::Unreg()
+{
+	LOGSTART("fileItemMgmt::Unreg");
+
+	if(!m_ptr)
+		return;
 
 	lockguard managementLock(mapLck);
-	setLockGuard;
 
-	if(m_globRef == mapItems.end())
-	{
-		// not globally registered, just ignore
+	// invalid or not globally registered?
+	if(m_ptr->m_globRef == mapItems.end())
 		return;
-	}
 
-	m_globRef->second.first--;
-	if (m_globRef->second.first <= 0)
+	lockguard fitemLock(*m_ptr);
+
+	if ( -- m_ptr->usercount <= 0)
 	{
-		// use the delay timer only if late expiration is needed and it isn't expired already
-		if (m_nTimeExpireAt && GetTime() < m_nTimeExpireAt)
+		if(m_ptr->m_status < fileitem::FIST_COMPLETE && m_ptr->m_status != fileitem::FIST_INITED)
 		{
-			m_nEarliestExpiration = MYSTD::min(m_nTimeExpireAt, m_nEarliestExpiration);
-			g_victor.RunLatestAt(m_nEarliestExpiration);
+			ldbg("usercount dropped to zero while downloading?: " << m_ptr->m_status);
+		}
+
+		// some file items will be held ready for some time
+		time_t when(0);
+		if (MAXTEMPDELAY && m_ptr->m_bCheckFreshness &&
+				m_ptr->m_status == fileitem::FIST_COMPLETE &&
+				((when=m_ptr->m_nTimeDlStarted+MAXTEMPDELAY) > GetTime()))
+		{
+			g_victor.ScheduleFor(when, cleaner::TYPE_EXFILEITEM);
 			return;
 		}
 
+		// nothing, let's put the item into shutdown state
+		m_ptr->m_status = fileitem::FIST_DLSTOP;
+		m_ptr->m_head.frontLine="HTTP/1.1 500 Cache file item expired";
+		m_ptr->notifyAll();
+
 		LOG("*this is last entry, deleting dl/fi mapping");
-		m_status = FIST_ERRNOUSER;
-		notifyAll();
-		mapItems.erase(m_globRef);
-		m_globRef = mapItems.end();
+		mapItems.erase(m_ptr->m_globRef);
+		m_ptr->m_globRef = mapItems.end();
 	}
 }
 
 
-tFileItemPtr fileitem::GetRegisteredFileItem(cmstring &sPathKey, bool bConsiderAltStore)
+fileItemMgmt fileItemMgmt::GetRegisteredFileItem(cmstring &sPathUnescaped, bool bConsiderAltStore)
 {
-	LOGSTART2s("fileitem::GetFileItem", sPathKey);
+	LOGSTART2s("fileitem::GetFileItem", sPathUnescaped);
 
 	MYTRY
 	{
-		fileitem *p = new fileitem(sPathKey); // need one because it will convert the path string to the proper value needed as key
-		tFileItemPtr sp(p);
+		mstring sPathRel(fileitem_with_storage::NormalizePath(sPathUnescaped));
 		lockguard lockGlobalMap(mapLck);
-
-		tFiGlobMap::iterator it=mapItems.find(p->m_sPathRel);
+		tFiGlobMap::iterator it=mapItems.find(sPathRel);
 		if(it!=mapItems.end())
 		{
 			if (bConsiderAltStore)
@@ -778,128 +765,126 @@ tFileItemPtr fileitem::GetRegisteredFileItem(cmstring &sPathKey, bool bConsiderA
 				// detect items that got stuck somehow
 				time_t now(GetTime());
 				time_t extime(now - acfg::stucksecs);
-				if (it->second.second.get()->m_nTimeDlDone < extime)
+				if (it->second->m_nTimeDlDone < extime)
 				{
 					// try to find its sibling which is in good state?
-					for (; it!=mapItems.end() && it->first == p->m_sPathRel; ++it)
+					for (; it!=mapItems.end() && it->first == sPathRel; ++it)
 					{
-						if (it->second.second.get()->m_nTimeDlDone >= extime)
+						if (it->second->m_nTimeDlDone >= extime)
 						{
-							it->second.first++;
+							it->second->usercount++;
 							LOG("Sharing an existing REPLACEMENT file item");
-							return it->second.second;
+							fileItemMgmt ret;
+							ret.m_ptr = it->second;
+							return ret;
 						}
 					}
 					// ok, then create a modded name version in the replacement directory
-					mstring sKeyPathOrig(p->m_sPathRel);
-					replaceChars(p->m_sPathRel, "/\\", '_');
-					p->m_sPathRel.insert(0, sReplDir + ltos(getpid()) + "_" + ltos(now))+"_";
-					p->m_sPathAbs = acfg::cacheDirSlash+p->m_sPathRel;
-
+					mstring sPathRelMod(sPathRel);
+					replaceChars(sPathRelMod, "/\\", '_');
+					sPathRelMod.insert(0, sReplDir + ltos(getpid()) + "_" + ltos(now)+"_");
 					LOG("Registering a new REPLACEMENT file item...");
-					p->m_globRef = mapItems.insert(make_pair(sKeyPathOrig, tFileRefEntry(1,sp)));
-					return sp;
+					fileitem_with_storage *p = new fileitem_with_storage(sPathRelMod);
+					p->usercount=1;
+					tFileItemPtr sp(p);
+					p->m_globRef = mapItems.insert(make_pair(sPathRel, sp));
+					fileItemMgmt ret;
+					ret.m_ptr = sp;
+					return ret;
 				}
 			}
 			LOG("Sharing existing file item");
-			it->second.first++;
-			return it->second.second;
+			it->second->usercount++;
+			fileItemMgmt ret;
+			ret.m_ptr = it->second;
+			return ret;
 		}
 		LOG("Registering the NEW file item...");
-		p->m_globRef = mapItems.insert(make_pair(p->m_sPathRel, tFileRefEntry(1,sp)));
-		return sp;
+		fileitem_with_storage *p = new fileitem_with_storage(sPathRel);
+		p->usercount=1;
+		tFileItemPtr sp(p);
+		p->m_globRef = mapItems.insert(make_pair(sPathRel, sp));
+		//lockGlobalMap.unLock();
+		fileItemMgmt ret;
+		ret.m_ptr = sp;
+		return ret;
 	}
 	MYCATCH(MYSTD::bad_alloc&)
 	{
 	}
-	return tFileItemPtr();
+	return fileItemMgmt();
 }
 
 // make the fileitem globally accessible
-bool fileitem::RegisterFileItem(SHARED_PTR<fileitem> spCustomFileItem)
+fileItemMgmt fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
 {
 	if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
-		return false;
+		return fileItemMgmt();
 
 	lockguard lockGlobalMap(mapLck);
 
 	if(ContHas(mapItems, spCustomFileItem->m_sPathRel))
-		return false; // conflict, another agent is already active
+		return fileItemMgmt(); // conflict, another agent is already active
 
 	spCustomFileItem->m_globRef = mapItems.insert(make_pair(spCustomFileItem->m_sPathRel,
-			tFileRefEntry(1, spCustomFileItem)));
+			spCustomFileItem));
 
-	return true;
+	spCustomFileItem->usercount=1;
+	fileItemMgmt ret;
+	ret.m_ptr = spCustomFileItem;
+	return ret;
 }
 
 
 // this method is supposed to be awaken periodically and detect items with ref count manipulated by
 // the request storm prevention mechanism. Items shall be be dropped after some time if no other
 // thread but us is using them.
-time_t fileitem::DoDelayedUnregAndCheck()
+time_t fileItemMgmt::BackgroundCleanup()
 {
+	LOGSTART2s("fileItemMgmt::BackgroundCleanup", GetTime());
 	lockguard lockGlobalMap(mapLck);
 	tFiGlobMap::iterator it, here;
 
 	time_t now=GetTime();
-
-	if(now<m_nEarliestExpiration) // unreachable anyhow
-		return m_nEarliestExpiration;
-
-	m_nEarliestExpiration = END_OF_TIME; // the correct time will be found below
+	time_t oldestGet = END_OF_TIME;
+	time_t expBefore = now - MAXTEMPDELAY;
 
 	for(it=mapItems.begin(); it!=mapItems.end();)
 	{
 		here=it++;
 
-		// not needed or not ready yet
-		if (!here->second.second->m_nTimeExpireAt)
-			continue;
-
-		m_nEarliestExpiration=min(here->second.second->m_nTimeExpireAt, m_nEarliestExpiration);
-
-		if (now < here->second.second->m_nTimeExpireAt)
-			continue;
-
 		// became busy again? Ignore this entry, it's new master will take care of the deletion ASAP
-		if(here->second.first>0)
+		if(here->second->usercount >0)
 			continue;
 
-		// ok, unused and delay is over. Destroy with the same sequence as Unreg does, care about
-		// live time.
-		tFileItemPtr local_ptr(here->second.second);
-		fileitem *p = static_cast<fileitem*>(local_ptr.get());
+		// find and ignore (but remember) the candidate(s) for the next cycle
+		if (here->second->m_nTimeDlStarted > expBefore)
 		{
-			lockguard g(p);
-			p->m_status = FIST_ERRNOUSER;
-			p->m_globRef = mapItems.end();
+			oldestGet = MYSTD::min(here->second->m_nTimeDlStarted, oldestGet);
+			continue;
+		}
+
+		// ok, unused and delay is over. Destroy with the same sequence as mgmt destructor does,
+		// care about life time.
+		tFileItemPtr local_ptr(here->second);
+		{
+			lockguard g(*local_ptr);
+			local_ptr->m_status = fileitem::FIST_DLSTOP;
+			local_ptr->m_globRef = mapItems.end();
 			mapItems.erase(here);
 		}
-		// p still alive, through local_ptr or external references to it
-		p->notifyAll();
-
+		local_ptr->notifyAll();
 	}
-	return m_nEarliestExpiration;
+
+	if(oldestGet == END_OF_TIME)
+		return oldestGet;
+
+	ldbg(oldestGet);
+
+	// preserving a few seconds to catch more of them in the subsequent run
+	return MYSTD::max(oldestGet + MAXTEMPDELAY, GetTime()+8);
 }
 
-#ifdef DEBUG
-
-void DumpItems()
-{
-/*	lockguard lockGlobalMap(mapLck);
-
-	map<string, tFileRefEntry>::iterator it=mapItems.begin();
-	cout << "Remaining file items:\n";
-	for(;it!=mapItems.end(); it++)
-	{
-		cout << it->second.ref->status << ": " << it->first <<endl;
-	}
-	*/
-}
-#endif
-
-
-
 ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos, size_t count)
 {
 #ifndef HAVE_LINUX_SENDFILE
@@ -914,58 +899,63 @@ ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos,
 #endif
 }
 
-void dump_handler(int) {
-	fileitem::dump_status();
-}
-
-void fileitem::dump_status()
+void fileItemMgmt::dump_status()
 {
 	tSS fmt;
-	aclog::err("File descriptor table:\n", NULL);
+	aclog::err("File descriptor table:\n");
 	for(tFiGlobMap::iterator it=mapItems.begin(); it!=mapItems.end(); ++it)
 	{
 		fmt.clear();
-		fmt << "FREF: " << it->first << " [" << it->second.first << "]:\n";
-		if(! it->second.second)
+		fmt << "FREF: " << it->first << " [" << it->second->usercount << "]:\n";
+		if(! it->second)
 		{
 			fmt << "\tBAD REF!\n";
 			continue;
 		}
 		else
 		{
-			fmt << "\t" << it->second.second->m_sPathRel
-					<< "\n\t" << it->second.second->m_sPathAbs
-					<< "\n\tPos.dl.agents: " << it->second.second->m_nDlRefsCount
-					<< "\n\tState: " << it->second.second->m_status
-					<< "\n\tPos: " << it->second.second->m_nIncommingCount << " , "
-					<< it->second.second->m_nRangeLimit << " , "
-					<< it->second.second->m_nSizeChecked << " , "
-					<< it->second.second->m_nSizeSeen
-			<< "\n\tExp: " << it->second.second->m_nEarliestExpiration << " , "
-			<< it->second.second->m_nTimeExpireAt << "\n\n";
+			fmt << "\t" << it->second->m_sPathRel
+					<< "\n\tDlRefCount: " << it->second->m_nDlRefsCount
+					<< "\n\tState: " << it->second->m_status
+					<< "\n\tFilePos: " << it->second->m_nIncommingCount << " , "
+					<< it->second->m_nRangeLimit << " , "
+					<< it->second->m_nSizeChecked << " , "
+					<< it->second->m_nSizeSeen
+					<< "\n\tGotAt: " << it->second->m_nTimeDlStarted << "\n\n";
 		}
 		aclog::err(fmt.c_str(), NULL);
 	}
+	aclog::flush();
 }
 
 fileitem_with_storage::~fileitem_with_storage()
 {
 	if(startsWith(m_sPathRel, sReplDir))
 	{
-		unlink(m_sPathAbs.c_str());
-		unlink((m_sPathAbs+".head").c_str());
+		::unlink(SZABSPATH(m_sPathRel));
+		unlink((SABSPATH(m_sPathRel)+".head").c_str());
 	}
 }
 
-#endif // MINIBUILD
-
-FiStatus fileitem_base::WaitForFinish(int *httpCode)
+fileItemMgmt::fileItemMgmt(const fileItemMgmt &src)
 {
-	setLockGuard;
-	while(m_status<FIST_COMPLETE)
-		wait();
-	if(httpCode)
-		*httpCode=m_head.getStatus();
-	return m_status;
+	fileItemMgmt *x = const_cast<fileItemMgmt *>(&src);
+	this->m_ptr = x->m_ptr;
+	x->m_ptr.reset();
 }
 
+fileItemMgmt& fileItemMgmt::operator=(const fileItemMgmt &src)
+{
+	fileItemMgmt *x = const_cast<fileItemMgmt *>(&src);
+	this->m_ptr = x->m_ptr;
+	x->m_ptr.reset();
+	return *this;
+}
+
+void fileItemMgmt::ReplaceWithLocal(fileitem* replacement)
+{
+	Unreg();
+	m_ptr.reset(replacement);
+}
+
+#endif // MINIBUILD
