@@ -238,7 +238,7 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 		return false;
 
 	// LFS on 32bit? That's not good for mmap. Don't risk incorrect behaviour.
-	if(uint64_t(statbuf.st_size) >  MYSTD::numeric_limits<size_t>::max())
+	if(uint64_t(statbuf.st_size) >  MAX_VAL(size_t))
     {
         errno=EFBIG;
         return false;
