@@ -20,9 +20,6 @@ using namespace MYSTD;
 
 #include <errno.h>
 
-// XXX no longer needed after debug logging overhaul?
-#define REPLEVEL (m_type==rechecks::FILE_INDEX ? 4 : (m_type==rechecks::FILE_PKG ? 5 : SPAMLEVEL))
-
 #if 0 // defined(DEBUG)
 #define CHUNKDEFAULT true
 #warning testing chunk mode
@@ -30,6 +27,8 @@ using namespace MYSTD;
 #define CHUNKDEFAULT false
 #endif
 
+mstring sHttp11("HTTP/1.1");
+
 #define SPECIAL_FD -42
 inline bool IsValidFD(int fd) { return fd>=0 || SPECIAL_FD == fd; }
 
@@ -39,7 +38,7 @@ inline bool IsValidFD(int fd) { return fd>=0 || SPECIAL_FD == fd; }
  * harddisk. Instead, it uses the download buffer and lets job object send the data straight from
  * it to the client.
  */
-class tPassThroughFitem : public fileitem_base
+class tPassThroughFitem : public fileitem
 {
 protected:
 
@@ -47,9 +46,10 @@ protected:
 	size_t m_nConsumable, m_nConsumed;
 
 public:
-	tPassThroughFitem(MYSTD::string s) : fileitem_base(s),
+	tPassThroughFitem(MYSTD::string s) :
 	m_pData(NULL), m_nConsumable(0), m_nConsumed(0)
 	{
+		m_sPathRel = s;
 		m_bAllowStoreData=false;
 		m_nSizeChecked = m_nSizeSeen = 0;
 	};
@@ -58,20 +58,9 @@ public:
 		m_nSizeChecked = m_nSizeSeen = 0;
 		return m_status = FIST_INITED;
 	}
-	virtual void Unreg() {
-		setLockGuard;
-		if(FIST_DLRECEIVING == m_status)
-		{
-			// no shit, there is some input waiting? bring the sender down ASAP
-			// this is not nice to the sender because it triggers a reconnection, but
-			// it's a local source so it shouldn't hurt much
-			m_status = FIST_ERRNOUSER;
-			notifyAll();
-		}
-	};
 	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
-	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *,
-			bool)
+	virtual bool DownloadStartedStoreHeader(const header & h, const char *,
+			bool, bool&)
 	{
 		setLockGuard;
 		m_head=h;
@@ -90,7 +79,7 @@ public:
 		m_nIncommingCount += size;
 
 		dbgline;
-		if (m_status >= FIST_LOCALERROR || m_status < FIST_DLGOTHEAD)
+		if (m_status > fileitem::FIST_COMPLETE || m_status < FIST_DLGOTHEAD)
 			return false;
 
 		if (size == 0)
@@ -103,12 +92,12 @@ public:
 			m_pData = data;
 			m_nConsumable=size;
 			m_nConsumed=0;
-			while(0 == m_nConsumed && m_status<FIST_LOCALERROR)
+			while(0 == m_nConsumed && m_status <= FIST_COMPLETE)
 				wait();
 
 			dbgline;
 			// let the downloader abort?
-			if(m_status >= FIST_LOCALERROR)
+			if(m_status >= FIST_DLERROR)
 				return false;
 
 			dbgline;
@@ -122,12 +111,12 @@ public:
 	{
 		setLockGuard;
 
-		while(0 == m_nConsumable && m_status<FIST_LOCALERROR
+		while(0 == m_nConsumable && m_status<=FIST_COMPLETE
 				&& ! (m_nSizeChecked==0 && m_status==FIST_COMPLETE))
 		{
 			wait();
 		}
-		if (m_status >= FIST_LOCALERROR || !m_pData)
+		if (m_status >= FIST_DLERROR || !m_pData)
 			return -1;
 
 		if(!m_nSizeChecked)
@@ -138,7 +127,8 @@ public:
 			r = 0;
 		if(r<0)
 		{
-			m_status=FIST_LOCALERROR;
+			m_status=FIST_DLERROR;
+			m_head.frontLine="HTTP/1.1 500 Data passing error";
 		}
 		else if(r>0)
 		{
@@ -152,18 +142,17 @@ public:
 	}
 };
 
-// fileitem replacement with custom storage for local data
-class tGeneratedFitemBase : public fileitem_base
+// base class for a fileitem replacement with custom storage for local data
+class tGeneratedFitemBase : public fileitem
 {
 public:
-	virtual void Unreg() {}; // NOOP, since this item is not shared
 	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
 	tSS m_data;
 
-	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg)
-	: fileitem_base(sFitemId), m_data(256)
+	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg) : m_data(256)
 	{
 		m_status=FIST_COMPLETE;
+		m_sPathRel=sFitemId;
 		m_head.type = header::ANSWER;
 		m_head.frontLine = "HTTP/1.1 ";
 		m_head.frontLine += (szFrontLineMsg ? szFrontLineMsg : "500 Internal Failure");
@@ -171,7 +160,7 @@ public:
 	}
 	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow)
 	{
-		if (m_status >= FIST_LOCALERROR || out_fd<0)
+		if (m_status > FIST_COMPLETE || out_fd<0)
 			return -1;
 		int r = m_data.syswrite(out_fd, nMax2SendNow);
 		if(r>0) nSendPos+=r;
@@ -184,7 +173,7 @@ public:
 		m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
 	}
 	// never used to store data
-	SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &, const char *, bool) {return false;};
+	bool DownloadStartedStoreHeader(const header &, const char *, bool, bool&) {return false;};
 	bool StoreFileData(const char *, unsigned int) {return false;};
 	header & HeadRef() { return m_head; }
 };
@@ -251,7 +240,6 @@ job::job(header *h, con *pParent) :
 	m_nChunkRemainingBytes(0),
 	m_type(rechecks::FILE_INVALID),
 	m_nReqRangeFrom(-1), m_nReqRangeTo(-1)
-//,m_nRedirRemaining(acfg::redirmax)
 {
 	LOGSTART2("job::job", "job creating, " << m_pReqHead->frontLine << " and this: " << uintptr_t(this));
 }
@@ -270,32 +258,16 @@ job::~job()
 			m_nAllDataCount, bErr);
 	
 	checkforceclose(m_filefd);
-	if(m_pItem)
-		m_pItem->Unreg();
 	delete m_pReqHead;
 }
 
-const string & GetMimeType(const string &path)
-{
-	tStrPos dpos = path.find_last_of('.');
-	if (dpos != stmiss)
-	{
-		NoCaseStringMap::const_iterator it = acfg::mimemap.find(path.substr(
-				dpos + 1));
-		if (it != acfg::mimemap.end())
-			return it->second;
-	}
-	static string nix("");
-	return nix;
-}
-
 
 inline void job::HandleLocalDownload(const string &visPath,
 		const string &fsBase, const string &fsSubpath)
 {
-	struct stat stbuf;
 	string absPath = fsBase+SZPATHSEP+fsSubpath;
-	if (0!=::stat(absPath.c_str(), &stbuf))
+	Cstat stbuf(absPath);
+	if (!stbuf)
 	{
 		switch(errno)
 		{
@@ -361,7 +333,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 					seal();
 				}
 			};
-			m_pItem.reset(static_cast<fileitem_base*>(new dirredirect(visPath)));
+			m_pItem.ReplaceWithLocal(new dirredirect(visPath));
 			return;
 		}
 
@@ -375,7 +347,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 			}
 		};
 		listing *p=new listing(visPath);
-		m_pItem.reset(static_cast<fileitem_base*>(p)); // assign to smart pointer ASAP, operations might throw
+		m_pItem.ReplaceWithLocal(p); // assign to smart pointer ASAP, operations might throw
 		tSS & page = p->m_data;
 
 		page << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"
@@ -407,7 +379,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 				tSS line;
 				if(bDir)
 					line << "[DIR]";
-				else if(startsWithSz(GetMimeType(dp.d_name), "image/"))
+				else if(startsWithSz(acfg::GetMimeType(dp.d_name), "image/"))
 					line << "[IMG]";
 				else
 					line << "[&nbsp;&nbsp;&nbsp;]";
@@ -436,28 +408,38 @@ inline void job::HandleLocalDownload(const string &visPath,
 	}
 	/*
 	 * This variant of file item handler sends a local file. The
-	 * header data is generated as needed.
+	 * header data is generated as needed, the relative cache path variable
+	 * is reused for the real path.
 	 */
 	class tLocalGetFitem : public fileitem_with_storage
 	{
 	public:
-		tLocalGetFitem(string s, string sLocalPath, struct stat &stdata) : fileitem_with_storage(s)
+		tLocalGetFitem(string sLocalPath, struct stat &stdata) :
+			fileitem_with_storage(sLocalPath)
 		{
 			m_bAllowStoreData=false;
-			m_sPathAbs=sLocalPath;
 			m_status=FIST_COMPLETE;
 			m_nSizeChecked=m_nSizeSeen=stdata.st_size;
 			m_bCheckFreshness=false;
 			m_head.type=header::ANSWER;
 			m_head.frontLine="HTTP/1.1 200 OK";
 			m_head.set(header::CONTENT_LENGTH, stdata.st_size);
-			const string &sMimeType=GetMimeType(sLocalPath);
+			cmstring &sMimeType=acfg::GetMimeType(sLocalPath);
 			if(!sMimeType.empty())
 				m_head.set(header::CONTENT_TYPE, sMimeType);
 		};
-		virtual void Unreg() {}; // NOOP, since this item is not shared
+		virtual int GetFileFd()
+		{
+			int fd=open(m_sPathRel.c_str(), O_RDONLY);
+		#ifdef HAVE_FADVISE
+			// optional, experimental
+			if(fd>=0)
+				posix_fadvise(fd, 0, m_nSizeChecked, POSIX_FADV_SEQUENTIAL);
+		#endif
+			return fd;
+		}
 	};
-	m_pItem.reset(static_cast<fileitem_base*>(new tLocalGetFitem(visPath, absPath, stbuf)));
+	m_pItem.ReplaceWithLocal(new tLocalGetFitem(absPath, stbuf));
 }
 
 inline bool job::ParseRange()
@@ -497,53 +479,54 @@ void job::PrepareDownload() {
 
     LOGSTART("job::PrepareDownload");
     
-#ifdef DEBUG
+#ifdef DEBUGLOCAL
     acfg::localdirs["stuff"]="/tmp/stuff";
     aclog::err(m_pReqHead->ToString());
 #endif
 
-    string sRawUriPath, sPathResdiual;
+    string sReqPath, sPathResidual;
     tHttpUrl tUrl; // parsed URL
 
     const acfg::tRepoData * pBackends(NULL); // appropriate backends
     const string * psVname(NULL); // virtual name for storage pool, if available
     
-    FiStatus fistate(FIST_FRESH);
+    fileitem::FiStatus fistate(fileitem::FIST_FRESH);
     bool bPtMode(false);
     bool bForceFreshnessChecks(false); // force update of the file, i.e. on dynamic index files?
-	tStrVec tokens;
-    
+    tSplitWalk tokenizer(& m_pReqHead->frontLine, SPACECHARS);
+
     if(m_pReqHead->type!=header::GET && m_pReqHead->type!=header::HEAD)
     	goto report_invpath;
+    if(!tokenizer.Next() || !tokenizer.Next()) // at path...
+    	goto report_invpath;
+    UrlUnescapeAppend(tokenizer, sReqPath);
+    if(!tokenizer.Next()) // at proto
+    	goto report_invpath;
+    m_bIsHttp11 = (sHttp11 == tokenizer.str());
     
-	if (3 != Tokenize(m_pReqHead->frontLine, SPACECHARS, tokens))
-		goto report_invpath;
+    USRDBG( "Decoded request URI: " << sReqPath);
 
-	sRawUriPath = tokens[1];
-	USRDBG(6, "Raw request URI: " << sRawUriPath);
-
-	m_bIsHttp11=(tokens[2] == "HTTP/1.1");
-	// a sane default? close for http 1.0, keep for 1.1.
-	// But if set by the client, comply!
+	// a sane default? normally, close-mode for http 1.0, keep for 1.1.
+	// But if set by the client then just comply!
 	m_bClientWants2Close =!m_bIsHttp11;
 	if(m_pReqHead && m_pReqHead->h[header::CONNECTION])
-		m_bClientWants2Close = 0==strncasecmp(m_pReqHead->h[header::CONNECTION], "close", 5);
+		m_bClientWants2Close = !strncasecmp(m_pReqHead->h[header::CONNECTION], "close", 5);
 
     // "clever" file system browsing attempt?
-	if(rechecks::Match(sRawUriPath, rechecks::NASTY_PATH)
-			|| stmiss != sRawUriPath.find("/_actmp")
-			|| startsWithSz(sRawUriPath, "/_"))
+	if(rechecks::Match(sReqPath, rechecks::NASTY_PATH)
+			|| stmiss != sReqPath.find("/_actmp")
+			|| startsWithSz(sReqPath, "/_"))
 		goto report_notallowed;
 
     MYTRY
 	{
 
-		if (0==sRawUriPath.compare(0, 12, "apt-cacher?/"))
-		sRawUriPath.erase(0, 12);
-		if (0==sRawUriPath.compare(0, 11, "apt-cacher/"))
-		sRawUriPath.erase(11);
+		if (0==sReqPath.compare(0, 12, "apt-cacher?/"))
+		sReqPath.erase(0, 12);
+		if (0==sReqPath.compare(0, 11, "apt-cacher/"))
+		sReqPath.erase(11);
 		
-		if(!tUrl.SetHttpUrl(sRawUriPath))
+		if(!tUrl.SetHttpUrl(sReqPath))
 		{
 			m_sMaintCmd="/";
 			return;
@@ -563,8 +546,8 @@ void job::PrepareDownload() {
 		if( (acfg::pUserPorts && ! acfg::pUserPorts->test(nPort)) // alarm
 				|| (!acfg::pUserPorts && nPort != 80))
 		{
-			if(acfg::port == tUrl.sPort) // admin failed to RTFM?
-				goto report_doubleproxy;
+			/*if(acfg::port == tUrl.sPort) // admin failed to RTFM?
+				goto report_doubleproxy;*/
 			goto report_invport;
 		}
 
@@ -572,16 +555,16 @@ void job::PrepareDownload() {
 		for(tStrPos pos; stmiss != (pos = tUrl.sPath.find("//")); )
 			tUrl.sPath.erase(pos, 1);
 
-		bPtMode=rechecks::MatchUncacheableRequest(tUrl.ToURI());
+		bPtMode=rechecks::MatchUncacheableRequest(tUrl.ToURI(false));
 
-		LOG("input uri: "<<tUrl.ToURI()<<" , dontcache-flag? " << bPtMode);
+		LOG("input uri: "<<tUrl.ToURI(false)<<" , dontcache-flag? " << bPtMode);
 
 		tStrPos nRepNameLen=acfg::reportpage.size();
 		if(nRepNameLen>0)
 		{
-			if(0==tUrl.sHost.compare(0, nRepNameLen, acfg::reportpage))
+			if(startsWith(tUrl.sHost, acfg::reportpage))
 			{
-				m_sMaintCmd=tUrl.sHost;
+				m_sMaintCmd=sReqPath;
 				return;
 			}
 			if (tUrl.sHost == "style.css")
@@ -594,14 +577,13 @@ void job::PrepareDownload() {
 
 		using namespace rechecks;
 
-		for(tStrMap::const_iterator it=acfg::localdirs.find(tUrl.sHost);
-				it != acfg::localdirs.end() ; )
 		{
-			string realSubPath;
-			if(! UrlDecode(tUrl.sPath, realSubPath) || Match(realSubPath, NASTY_PATH))
-				goto report_invpath; // what the...
-			HandleLocalDownload(sRawUriPath, it->second, realSubPath);
-			return;
+			tStrMap::const_iterator it = acfg::localdirs.find(tUrl.sHost);
+			if (it != acfg::localdirs.end())
+			{
+				HandleLocalDownload(sReqPath, it->second, tUrl.sPath);
+				return;
+			}
 		}
 
 		// entered directory but not defined as local? Then 404 it with hints
@@ -618,19 +600,19 @@ void job::PrepareDownload() {
 			goto report_notallowed;
 		
 		// got something valid, has type now, trace it
-		USRDBG(REPLEVEL, "Processing new job, "<<m_pReqHead->frontLine);
+		USRDBG("Processing new job, "<<m_pReqHead->frontLine);
 
 		// resolve to an internal location
-		psVname = acfg::GetRepNameAndPathResidual(tUrl, sPathResdiual);
+		psVname = acfg::GetRepNameAndPathResidual(tUrl, sPathResidual);
 		
 		if(psVname)
-			m_sFileLoc=*psVname+SZPATHSEP+sPathResdiual;
+			m_sFileLoc=*psVname+SZPATHSEP+sPathResidual;
 		else
 			m_sFileLoc=tUrl.sHost+tUrl.sPath;
 
 		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type == FILE_INDEX);
 
-    	m_pItem=fileitem::GetRegisteredFileItem(m_sFileLoc, bForceFreshnessChecks);
+    	m_pItem=fileItemMgmt::GetRegisteredFileItem(m_sFileLoc, bForceFreshnessChecks);
 
 	}
 	MYCATCH(MYSTD::out_of_range&) // better safe...
@@ -640,27 +622,28 @@ void job::PrepareDownload() {
     
     if(!m_pItem)
     {
-    	USRDBG(nix, "Error creating file item for " << m_sFileLoc);
+    	USRDBG("Error creating file item for " << m_sFileLoc);
     	goto report_overload;
     }
     
     fistate = m_pItem->Setup(bForceFreshnessChecks);
 	LOG("Got initial file status: " << fistate);
 
-	if (bPtMode && fistate != FIST_COMPLETE)
+	if (bPtMode && fistate != fileitem::FIST_COMPLETE)
 		fistate = _SwitchToPtItem(m_sFileLoc);
 
 	ParseRange();
 
-	if(fistate==FIST_COMPLETE)
+	if(fistate==fileitem::FIST_COMPLETE)
 		return; // perfect, done here
 
 	// early optimization for requests which want a defined file part which we already have
 	// no matter whether the file is complete or not
 	// handles different cases: GET with range, HEAD with range, HEAD with no range
-	if((m_nReqRangeFrom>=0 && m_nReqRangeTo>=0) || (m_pReqHead->type==header::HEAD && 0!=(m_nReqRangeTo=-1)))
+	if((m_nReqRangeFrom>=0 && m_nReqRangeTo>=0)
+			|| (m_pReqHead->type==header::HEAD && 0!=(m_nReqRangeTo=-1)))
 	{
-		lockguard g(*m_pItem);
+		lockguard g(m_pItem.get().get());
 		if(m_pItem->CheckUsableRange_unlocked(m_nReqRangeTo))
 		{
 			LOG("Got a partial request for incomplete download; range is available");
@@ -674,12 +657,12 @@ void job::PrepareDownload() {
     	goto report_offlineconf;
     }
     dbgline;
-    if( fistate < FIST_DLGOTHEAD) // needs a downloader 
+    if( fistate < fileitem::FIST_DLGOTHEAD) // needs a downloader
     {
     	dbgline;
     	if(!m_pParentCon->SetupDownloader(m_pReqHead->h[header::XFORWARDEDFOR]))
     	{
-    		USRDBG(x, "Error creating download handler for "<<m_sFileLoc);
+    		USRDBG( "Error creating download handler for "<<m_sFileLoc);
     		goto report_overload;
     	}
     	
@@ -688,33 +671,33 @@ void job::PrepareDownload() {
 		{
 			if(psVname && NULL != (pBackends=acfg::GetBackendVec(*psVname)))
 			{
-				LOG("Backends found, using them with " << sPathResdiual
-						<< ", first backend: " <<pBackends->m_backends.front().ToURI());
+				LOG("Backends found, using them with " << sPathResidual
+						<< ", first backend: " <<pBackends->m_backends.front().ToURI(false));
 
 				if(! bPtMode && rechecks::
-						MatchUncacheableTarget(pBackends->m_backends.front().ToURI()+sPathResdiual))
+						MatchUncacheableTarget(pBackends->m_backends.front().ToURI(false)+sPathResidual))
 				{
 					fistate=_SwitchToPtItem(m_sFileLoc);
 				}
 
-				m_pParentCon->m_pDlClient->AddJob(m_pItem, pBackends, sPathResdiual);
+				m_pParentCon->m_pDlClient->AddJob(m_pItem.get(), pBackends, sPathResidual);
 			}
 			else
 			{
 			    if(acfg::forcemanaged)
 			    	goto report_notallowed;
-			    LOG("Passing new job for " << tUrl.ToURI() << " to " << m_pParentCon->m_dlerthr);
+			    LOG("Passing new job for " << tUrl.ToURI(false) << " to " << m_pParentCon->m_dlerthr);
 
-				if(! bPtMode && rechecks::MatchUncacheableTarget(tUrl.ToURI()))
+				if(! bPtMode && rechecks::MatchUncacheableTarget(tUrl.ToURI(false)))
 					fistate=_SwitchToPtItem(m_sFileLoc);
 
-			    m_pParentCon->m_pDlClient->AddJob(m_pItem, tUrl);
+			    m_pParentCon->m_pDlClient->AddJob(m_pItem.get(), tUrl);
 			}
 			ldbg("Download job enqueued for " << m_sFileLoc);
 		}
 		MYCATCH(MYSTD::bad_alloc&) // OOM, may this ever happen here?
 		{
-			USRDBG(y, "Out of memory");
+			USRDBG( "Out of memory");
 			goto report_overload;
 		};
 	}
@@ -726,8 +709,8 @@ report_overload:
     return ;
 
 report_notallowed:
-	SetErrorResponse((tSS() << "403 Forbidden file type or location: " << sRawUriPath).c_str());
-//    USRDBG(5, sRawUriPath + " -- ACCESS FORBIDDEN");
+	SetErrorResponse((tSS() << "403 Forbidden file type or location: " << sReqPath).c_str());
+//    USRDBG( sRawUriPath + " -- ACCESS FORBIDDEN");
     return ;
 
 report_offlineconf:
@@ -739,14 +722,14 @@ report_invpath:
     return ;
 
 report_invport:
-	SetErrorResponse("403 Prohibited port or config error (double proxy)");
+	SetErrorResponse("403 Configuration error (confusing proxy mode) or prohibited port (see AllowUserPorts)");
     return ;
-
+/*
 report_doubleproxy:
 	SetErrorResponse("403 URL seems to be made for proxy but contains apt-cacher-ng port. "
     		"Inconsistent apt configuration?");
     return ;
-
+*/
 }
 
 #define THROW_ERROR(x) { if(m_nAllDataCount) return R_DISCON; SetErrorResponse(x); return R_AGAIN; }
@@ -761,7 +744,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 	}
 	
 	off_t nGoodDataSize(0);
-	FiStatus fistate(FIST_LOCALERROR);
+	fileitem::FiStatus fistate(fileitem::FIST_DLERROR);
 	header respHead; // template of the response header, e.g. as found in cache
 
 	if (confd<0)
@@ -769,14 +752,14 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 
 	if (m_pItem)
 	{
-		lockguard g(*m_pItem);
+		lockguard g(m_pItem.get().get());
 		
 		for(;;)
 		{
 			fistate=m_pItem->GetStatusUnlocked(nGoodDataSize);
 			
 			LOG(fistate);
-			if (fistate>=FIST_LOCALERROR)
+			if (fistate > fileitem::FIST_COMPLETE)
 			{
 				const header &h = m_pItem->GetHeaderUnlocked();
 				g.unLock(); // item lock must be released in order to replace it!
@@ -788,7 +771,10 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				if(h.frontLine.length()>9 && h.getStatus()!=200)
 					SetErrorResponse(h.frontLine.c_str()+9, h.h[header::LOCATION]);
 				else // good ungood? confused somewhere?!
+				{
+					ldbg("good ungood, consused?" << h.frontLine)
 					SetErrorResponse("500 Unknown error");
+				}
 				return R_AGAIN;
 			}
 			/*
@@ -797,12 +783,12 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 			*/
 			if(m_bNoDownloadStarted)
 			{
-				fistate = FIST_DLRECEIVING;
+				fistate = fileitem::FIST_DLRECEIVING;
 				break;
 			}
 			// or wait for the dl source to get data at the position we need to start from
-			LOG(fistate << m_nSendPos << nGoodDataSize);
-			if(fistate==FIST_COMPLETE || (m_nSendPos < nGoodDataSize && fistate>=FIST_DLGOTHEAD))
+			LOG("sendstate: " << fistate << " , sendpos: " << m_nSendPos << nGoodDataSize);
+			if(fistate==fileitem::FIST_COMPLETE || (m_nSendPos < nGoodDataSize && fistate>=fileitem::FIST_DLGOTHEAD))
 				break;
 			
 			dbgline;
@@ -832,21 +818,23 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				case(STATE_SEND_MAIN_HEAD):
 				{
 					ldbg("STATE_FRESH");
-					if(fistate<FIST_DLGOTHEAD) // be sure about that
+					if(fistate < fileitem::FIST_DLGOTHEAD) // be sure about that
 						return R_AGAIN;
 					m_state=STATE_SEND_BUFFER;
 					m_backstate=STATE_HEADER_SENT; // could be changed while creating header
-
 					const char *szErr = BuildAndEnqueHeader(fistate, nGoodDataSize, respHead);
 					if(szErr) THROW_ERROR(szErr);
-					USRDBG(REPLEVEL, "Prepared response header for user: \n" << m_sendbuf );
+					USRDBG("Response header to be sent in the next cycle: \n" << m_sendbuf );
+#ifdef DEBUG
+#warning TODO: review the life cycle, might avoid one select call in the function above (prio:low)
+#endif
 					return R_AGAIN;
 				}
 				case(STATE_HEADER_SENT):
 				{
 					ldbg("STATE_HEADER_SENT");
 					
-					if( fistate < FIST_DLGOTHEAD)
+					if(fistate < fileitem::FIST_DLGOTHEAD)
 					{
 						ldbg("ERROR condition detected: starts activity while downloader not ready")
 						return R_AGAIN;
@@ -867,7 +855,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 #define GOTOENDE { m_state=STATE_FINISHJOB ; continue; }
 					if(m_nSendPos>=nGoodDataSize)
 					{
-						if(fistate>=FIST_COMPLETE)
+						if(fistate>=fileitem::FIST_COMPLETE)
 							GOTOENDE;
 						LOG("Cannot send more, not enough fresh data yet");
 						return R_AGAIN;
@@ -883,7 +871,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 
 					// shortcuts
 					if(m_nSendPos>m_nCurrentRangeLast ||
-							(fistate==FIST_COMPLETE && m_nSendPos==nGoodDataSize))
+							(fistate==fileitem::FIST_COMPLETE && m_nSendPos==nGoodDataSize))
 						GOTOENDE;
 					
 					if(n<0)
@@ -942,7 +930,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 						m_sendbuf.drop(r);
 						if(m_sendbuf.empty())
 						{
-							USRDBG(REPLEVEL, "Returning to last state, " << m_backstate);
+							USRDBG("Returning to last state, " << m_backstate);
 							m_state=m_backstate;
 							continue;
 						}
@@ -978,10 +966,12 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 		}
 		//ASSERT(!"UNREACHED");
 	}
+	//ASSERT(!"UNREACHEABLE");
+	return R_DISCON;
 }
 
 
-inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
+inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 		const off_t &nGooddataSize, header& respHead)
 {
 	LOGSTART("job::BuildAndEnqueHeader");
@@ -1060,11 +1050,11 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 					 * fetching from 0 to start position for many minutes. If the resumed position
 					 * is beyond of what we already have, fall back to 200 (complete download).
 					 */
-					if(  fistate==FIST_COMPLETE
+					if(  fistate==fileitem::FIST_COMPLETE
 							// or can start sending within this range (positive range-from)
 							|| (
-									fistate >= FIST_DLGOTHEAD
-									&& fistate <= FIST_COMPLETE
+									fistate >= fileitem::FIST_DLGOTHEAD
+									&& fistate <= fileitem::FIST_COMPLETE
 									&& nGooddataSize>=m_nReqRangeFrom
 								)
 							// don't care, found special hint from acngfs (kludge...)
@@ -1072,7 +1062,7 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 
 						)
 					{
-						USRDBG(20, "request range problem: ");
+						USRDBG("request range problem: ");
 
 						// detect errors, out-of-range case
 						if(m_nReqRangeFrom>=nContLen || m_nReqRangeTo<m_nReqRangeFrom)
@@ -1138,26 +1128,19 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 	return 0;
 }
 
-FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
+fileitem::FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
 {
 	// Changing to local pass-through file item
 	LOGSTART("job::_SwitchToPtItem");
 	// exception-safe sequence
-	tFileItemPtr tmp=m_pItem;
-	m_pItem.reset();
-	tmp->Unreg();
-	m_pItem.reset(new tPassThroughFitem(m_sFileLoc));
+	m_pItem.ReplaceWithLocal(new tPassThroughFitem(m_sFileLoc));
 	return m_pItem->Setup(true);
 }
 
 
 void job::SetErrorResponse(const char * errorLine, const char *szLocation)
 {
-	LOGSTART2("job::SetErrorResponse", errorLine);
-
-	if(m_pItem)
-		m_pItem->Unreg();
-
+	LOGSTART2("job::SetErrorResponse", errorLine << " ; for " << m_sOrigUrl);
 	class erroritem: public tGeneratedFitemBase
 	{
 	public:
@@ -1176,8 +1159,7 @@ void job::SetErrorResponse(const char * errorLine, const char *szLocation)
 
 	erroritem *p = new erroritem("noid", errorLine);
 	p->HeadRef().set(header::LOCATION, szLocation);
-	tFileItemPtr pNew(static_cast<fileitem_base*>(p));
-	m_pItem=pNew;
+	m_pItem.ReplaceWithLocal(p);;
 	//aclog::err(tSS() << "fileitem is now " << uintptr_t(m_pItem.get()));
 	m_state=STATE_SEND_MAIN_HEAD;
 }
