@@ -28,14 +28,12 @@ using namespace MYSTD;
 
 tWUIPage::tWUIPage(int fd) :
 	m_reportFD(fd),
-	m_szDecoFile(NULL),
-	m_fmtBuf(NULL), m_fmtSize(0)
+	m_szDecoFile(NULL)
 {
 }
 
 tWUIPage::~tWUIPage()
 {
-	free(m_fmtBuf);
 }
 
 bool tWUIPage::SendRawData(const char *data, size_t len, int flags)
@@ -60,7 +58,7 @@ bool tWUIPage::SendRawData(const char *data, size_t len, int flags)
 	return true;
 }
 
-void tWUIPage::SendChunk(const char *data, size_t len)
+void tWUIPage::SendChunk(const char *data, size_t len, bool bRemoteOnly)
 {
 	if(!data || !len || m_reportFD<0)
 		return;
@@ -76,6 +74,9 @@ void tWUIPage::SendChunk(const char *data, size_t len)
 	SendRawData(buf, l, MSG_MORE|MSG_NOSIGNAL);
 	SendRawData(data, len, MSG_MORE|MSG_NOSIGNAL);
 	SendRawData("\r\n", 2, MSG_NOSIGNAL);
+
+	if(!bRemoteOnly)
+		AfterSendChunk(data, len);
 }
 
 void tWUIPage::EndTransfer() 
@@ -217,7 +218,7 @@ void DispatchAndRunMaintTask(cmstring &cmd, int conFD, const char * szAuthLine)
 			else // ok... just show the default page
 				goto l_show_cnc;
 		}
-		else if (cmd==acfg::reportpage)
+		else if (cmd.compare(1, acfg::reportpage.length(), acfg::reportpage) == 0)
 		{
 			l_show_cnc:
 			pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200 OK");
