@@ -18,8 +18,8 @@
 
 using namespace MYSTD;
 
-string sPathSep(SZPATHSEP);
-string sPathSepUnix(SZPATHSEPUNIX);
+cmstring sPathSep(SZPATHSEP);
+cmstring sPathSepUnix(SZPATHSEPUNIX);
 
 /*
 int getUUID() {
@@ -128,8 +128,6 @@ tStrVec::size_type Tokenize(const string & in, const char *sep,
 	return (out.size()-nBefore);
 }
 
-
-
 void StrSubst(string &contents, const string &from, const string &to)
 {
 	tStrPos pos;
@@ -152,41 +150,6 @@ void Join(MYSTD::string &out, const MYSTD::string & sep, const tStrVec & tokens)
 			
 }
 
-
-bool slow_ParseKeyValLine(const string & sIn, string & sOutKey, string & sOutVal)
-{
-	sOutVal.clear();
-	sOutKey.clear();
-	mstring *t = NULL;
-	
-	for(cmstring::const_iterator p=sIn.begin(); p!=sIn.end(); ++p)
-	{
-		if(t!=&sOutKey && strchr(SPACECHARS, UCHAR(*p))) // trim in key part and beginning of value
-			continue;
-		if(*p == ':')
-		{
-			if(sOutKey.empty())
-				return false;
-			t = &sOutVal;
-			continue;
-		}
-		if(!t)
-		{
-			if(*p == '#') // comment line mark
-				return false;
-
-			t = &sOutKey;
-		}
-		t->push_back(*p);
-	}
-	
-	bool ret = (t==&sOutVal && !sOutVal.empty());
-	trimBack(sOutVal);
-	return ret;
-}
-
-
-
 bool ParseKeyValLine(const string & sIn, string & sOutKey, string & sOutVal)
 {
 	// reuse the output string as buffer
@@ -220,12 +183,14 @@ bool ParseKeyValLine(const string & sIn, string & sOutKey, string & sOutVal)
 }
 
 
-bool tHttpUrl::SetHttpUrl(string url)
+bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
 {
 	sPort.clear();
 	sHost.clear();
 	sPath.clear();
 	
+	mstring url = UrlUnescape(sUrlRaw);
+
 	trimBack(url);
 	trimFront(url);
 		
@@ -307,9 +272,30 @@ bool tHttpUrl::SetHttpUrl(string url)
 	
 }
 
-string tHttpUrl::ToURI() const 
-{ 
-	return string("http://") + (sPort.empty()? (sHost+sPath) : (sHost+":"+sPort+sPath));
+string tHttpUrl::ToURI(bool bUrlEscaped) const
+{
+	string s("http://");
+	if (bUrlEscaped)
+	{
+		UrlEscapeAppend(sHost, s);
+		if (!sPort.empty())
+		{
+			s += ':';
+			s += sPort;
+		}
+		UrlEscapeAppend(sPath, s);
+	}
+	else
+	{
+		s += sHost;
+		if (!sPort.empty())
+		{
+			s += ':';
+			s += sPort;
+		}
+		s += sPath;
+	}
+	return s;
 }
 
 #if defined(HAVE_WORDEXP) || defined(HAVE_GLOB)
@@ -369,50 +355,8 @@ void MakeAbsolutePath(MYSTD::string &dirToFix, const MYSTD::string &reldir)
 }
 */
 
-void HttpEscape(string &s)
-{
-	tStrPos p;
-	while(stmiss!=(p=s.find_first_of("~ ")))
-	{
-		switch(s[p])
-		{
-		case('~'): s.replace(p, 1, "%7e"); break;
-		case(' '): s.replace(p, 1, "&nbsp"); break;
-		}
-	}
-}
-
 extern uint_fast16_t hexmap[];
 
-bool UrlDecode(const MYSTD::string &from, MYSTD::string & to)
-{
-	bool ret=true;
-	for(string::size_type i=0; i<from.length(); i++)
-	{
-		if(from[i] != '%')
-			to+=from[i];
-		else if(i>from.length()-3 // cannot start another sequence here (too short), keep as-is
-				|| hexmap[(UCHAR)from[i+1]]>15 // not hex?
-				|| hexmap[(UCHAR)from[i+2]]>15)
-		{
-			ret=false;
-		}
-		else
-		{
-			to+=char(16*hexmap[(unsigned char) from[i+1]] + hexmap[(unsigned char) from[i+2]]);
-			i+=2;
-		}
-	}
-	return ret;
-}
-
-mstring UrlDecode(cmstring &from)
-{
-	mstring ret; // let the compiler optimize
-	UrlDecode(from, ret);
-	return ret;
-}
-
 cmstring sEmptyString("");
 
 /*
@@ -463,18 +407,157 @@ bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength)
 	}
 	return true;
 }
+char h2t_map[] =
+	{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
+			'e', 'f' };
 
 string BytesToHexString(const uint8_t sum[], unsigned short lengthBin)
 {
 	string sRet;
-	char hexmap[] =
-	{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
-			'e', 'f' };
 	for (int i=0; i<lengthBin; i++)
 	{
-		sRet+=(char) hexmap[sum[i]>>4];
-		sRet+=(char) hexmap[sum[i]&0xf];
+		sRet+=(char) h2t_map[sum[i]>>4];
+		sRet+=(char) h2t_map[sum[i]&0xf];
 	}
-
 	return sRet;
 }
+
+
+inline bool is_safe_url_char(char c)
+{
+// follows rfc3986 except of ~ which some servers seem to handle incorrectly,
+	// https://alioth.debian.org/tracker/?func=detail&aid=314030&group_id=100566&atid=413111
+	switch (c)
+	{
+	case '/':
+	case '0':
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9':
+	case 'a':
+	case 'b':
+	case 'c':
+	case 'd':
+	case 'e':
+	case 'f':
+	case 'g':
+	case 'h':
+	case 'i':
+	case 'j':
+	case 'k':
+	case 'l':
+	case 'm':
+	case 'n':
+	case 'o':
+	case 'p':
+	case 'q':
+	case 'r':
+	case 's':
+	case 't':
+	case 'u':
+	case 'v':
+	case 'w':
+	case 'x':
+	case 'y':
+	case 'z':
+	case 'A':
+	case 'B':
+	case 'C':
+	case 'D':
+	case 'E':
+	case 'F':
+	case 'G':
+	case 'H':
+	case 'I':
+	case 'J':
+	case 'K':
+	case 'L':
+	case 'M':
+	case 'N':
+	case 'O':
+	case 'P':
+	case 'Q':
+	case 'R':
+	case 'S':
+	case 'T':
+	case 'U':
+	case 'V':
+	case 'W':
+	case 'X':
+	case 'Y':
+	case 'Z':
+	case '-':
+	case '.':
+	case '_':
+		return true;
+	default:
+		return false;
+	}
+}
+
+void UrlEscapeAppend(cmstring &s, mstring &sTarget)
+{
+	for(cmstring::const_iterator it=s.begin(); it!=s.end();++it)
+	{
+		if(is_safe_url_char(*it))
+			sTarget+=*it;
+		else
+		{
+			char buf[4] = { '%', h2t_map[uint8_t(*it) >> 4], h2t_map[uint8_t(*it) & 0x0f],'\0'};
+			sTarget+=buf;
+		}
+	}
+}
+
+mstring UrlEscape(cmstring &s)
+{
+	mstring ret;
+	UrlEscapeAppend(s, ret);
+	return ret;
+}
+mstring DosEscape(cmstring &s)
+{
+	mstring ret;
+	for(cmstring::const_iterator it=s.begin(); it!=s.end();++it)
+	{
+		if('/' == *it)
+			ret+=SZPATHSEP;
+		else if(is_safe_url_char(*it))
+			ret+=*it;
+		else
+		{
+			char buf[4] = { '%', h2t_map[uint8_t(*it) >> 4], h2t_map[uint8_t(*it) & 0x0f],'\0'};
+			ret += buf;
+		}
+	}
+	return ret;
+}
+
+bool UrlUnescapeAppend(cmstring &from, mstring & to)
+{
+	bool ret=true;
+	for(string::size_type i=0; i<from.length(); i++)
+	{
+		if(from[i] != '%')
+			to+=from[i];
+		else if(i>from.length()-3 // cannot start another sequence here (too short), keep as-is
+				|| hexmap[(UCHAR)from[i+1]]>15 // not hex?
+				|| hexmap[(UCHAR)from[i+2]]>15)
+		{
+			ret=false;
+		}
+		else
+		{
+			to+=char(16*hexmap[(unsigned char) from[i+1]] + hexmap[(unsigned char) from[i+2]]);
+			i+=2;
+		}
+	}
+	return ret;
+}
+
