@@ -60,14 +60,14 @@ bool pkgmirror::ProcessRegular(const string &sPath, const struct stat &stinfo)
 #ifdef DEBUG
 			pair<tStrSet::iterator,bool> res= m_pathFilter.insert(filtStr);
 			if(res.second)
-				SendFmt() << "target cand: " << filtStr << "<br>\n";
+				SendFmt << "target cand: " << filtStr << "<br>\n";
 #else
 			m_pathFilter.insert(filtStr);
 #endif
 		}
 	}
 
-	return ! CheckAbortCondition();
+	return ! CheckStopSignal();
 }
 
 tStrDeq GetVariants(cmstring &mine)
@@ -128,22 +128,22 @@ void pkgmirror::Action(const string &cmd)
 
 	DirectoryWalk(acfg::cachedir, this);
 
-	if(CheckAbortCondition())
+	if(CheckStopSignal())
 		return;
 
 	if(m_indexFilesRel.empty())
 	{
-		SendChunk("<font size=0 color=red>No index files detected. Unable to continue, cannot map files to internal locations.</font>");
+		SendChunk("<div class=\"ERROR\">No index files detected. Unable to continue, cannot map files to internal locations.</div>");
 		return;
 	}
 
-	if(CheckAbortCondition())
+	if(CheckStopSignal())
 		return;
 
 	if(!m_bSkipIxUpdate)
 		UpdateIndexFiles();
 
-	if(CheckAbortCondition())
+	if(CheckStopSignal())
 		return;
 
 	// prepare wildcard matching and collecting
@@ -234,7 +234,7 @@ void pkgmirror::Action(const string &cmd)
 
 	for (tStrSet::iterator it=srcs.begin(); it!=srcs.end(); it++)
 	{
-		SendFmt() << "File list: " << *it<< "<br>";
+		SendFmt << "File list: " << *it<< "<br>";
 
 		if(m_bSkipIxUpdate)
 			continue;
@@ -242,7 +242,7 @@ void pkgmirror::Action(const string &cmd)
 		if(!GetFlags(*it).uptodate)
 			Download(*it, true);
 
-		if(CheckAbortCondition())
+		if(CheckStopSignal())
 			return;
 	}
 
@@ -254,30 +254,30 @@ void pkgmirror::Action(const string &cmd)
 
 		UINT dcount=0;
 
-		SendFmt() << "<b>Counting downloadable content size..."
+		SendFmt << "<b>Counting downloadable content size..."
 				<< (m_bAsNeeded? " (filtered)" : "")  << "</b><br>";
 
 		for (tStrSet::iterator it = srcs.begin(); it != srcs.end(); it++)
 		{
 #ifdef DEBUG
 			if(LOG_MORE&acfg::debug)
-				SendFmt() << "mirror check: " << *it;
+				SendFmt << "mirror check: " << *it;
 #endif
 			off_t needBefore=(m_totalSize-m_totalHave);
 
 			ParseAndProcessIndexFile(*this, *it, GuessIndexTypeFromURL(*it));
 
-			SendFmt() << *it << ": "
+			SendFmt << *it << ": "
 					<< offttosH((m_totalSize-m_totalHave)-needBefore)
 					<< " to download<br>\n";
 
 			if(m_bUseDelta)
 				dcount+=ConfigDelta(*it);
 
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return;
 		}
-		SendFmt() << "Total size: " << offttosH(m_totalSize) << ", to download: about "
+		SendFmt << "Total size: " << offttosH(m_totalSize) << ", to download: about "
 				<< offttosH(m_totalSize-m_totalHave) << "<br>\n";
 
 		if(m_bUseDelta && !dcount)
@@ -286,13 +286,13 @@ void pkgmirror::Action(const string &cmd)
 
 	if(m_bDoDownload && (!m_bCalcSize || m_totalSize!=m_totalHave))
 	{
-		SendFmt() << "<b>Starting download...</b><br>";
+		SendFmt << "<b>Starting download...</b><br>";
 
 		m_bCalcSize=false;
 
 		for (tStrSet::iterator it=srcs.begin(); it!=srcs.end(); it++)
 		{
-			if(CheckAbortCondition())
+			if(CheckStopSignal())
 				return;
 
 			ConfigDelta(*it);
@@ -323,10 +323,10 @@ inline bool pkgmirror::ConfigDelta(cmstring &sPathRel)
 		if(acfg::debug & LOG_MORE)
 		{
 			if(!pRepo)
-				SendFmt() << "hm, no delta provider for " << sPathRel;
+				SendFmt << "hm, no delta provider for " << sPathRel;
 		}
 		else
-			SendFmt()<< "hm, delta for " << sPathRel << " is what? " <<pRepo->m_deltasrc.sHost;
+			SendFmt<< "hm, delta for " << sPathRel << " is what? " <<pRepo->m_deltasrc.sHost;
 #endif
 		if (pRepo && !pRepo->m_deltasrc.sHost.empty())
 			m_pDeltaSrc = &pRepo->m_deltasrc;
@@ -366,7 +366,7 @@ void pkgmirror::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressFor
 		if (pos != stmiss)
 			filter.append(entry.sFileName, 0, pos);
 //#ifdef DEBUG
-//		SendFmt() << "filter: " << filter << "<br>";
+//		SendFmt << "filter: " << filter << "<br>";
 //#endif
 
 		if (!ContHas(m_pathFilter, filter))
@@ -426,18 +426,18 @@ void pkgmirror::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressFor
 			sort(sorted.begin(), sorted.end(), CompDebVerLessThan);
 
 #ifdef DEBUG
-			SendFmt() << "Found " << sorted.size() << " from " << oldebs.size() << " debs suitable for patching<br>";
+			SendFmt << "Found " << sorted.size() << " from " << oldebs.size() << " debs suitable for patching<br>";
 #endif
 			while(!sorted.empty() && !bhaveit)
 			{
 				tSS uri, srcAbs;
-				uri << m_pDeltaSrc->ToURI() << entry.sDirectory.substr(m_repCutLen+1)
+				uri << m_pDeltaSrc->ToURI(false) << entry.sDirectory.substr(m_repCutLen+1)
 						<< parts[0] << "_"
 						<< sorted.back() << "_" <<
 						parts[1] << "_" << parts[2] << "delta";
 
 #ifdef DEBUG
-				SendFmt() << uri << "<br>\n";
+				SendFmt << uri << "<br>\n";
 #endif
 				srcAbs << CACHE_BASE << entry.sDirectory<< parts[0] << "_"
 						<< sorted.back() << "_"  << parts[2];
@@ -458,7 +458,7 @@ void pkgmirror::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressFor
 					::setenv("from", srcAbs.c_str(), true);
 					::setenv("to", SZABSPATH(TEMPRESULT), true);
 
-				SendFmt() << "Fetched: " << uri << "<br>\n";
+				SendFmt << "Fetched: " << uri << "<br>\n";
 				cerr << "debpatch " << getenv("delta") << " " << getenv("from")
 						<< " " << getenv("to");
 
@@ -496,7 +496,7 @@ void pkgmirror::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressFor
 						if(bhaveit)
 						{
 							off_t nPatchTransferSize = GetFileSize(sDeltaPathAbs, -123);
-							SendFmt() << "Rebuilt " << tgtRel << " using Debdelta "
+							SendFmt << "Rebuilt " << tgtRel << " using Debdelta "
 									<< "<i>(" << nPatchTransferSize / 1024
 									<< ":" << entry.fpr.size / 1024
 									<< "KiB)</i>\n<br>\n";
@@ -504,7 +504,7 @@ void pkgmirror::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressFor
 
 					}
 					else
-						SendFmt() << "Debpatch couldn't rebuild " << tgtRel<<"<br>\n";
+						SendFmt << "Debpatch couldn't rebuild " << tgtRel<<"<br>\n";
 				}
 			}
 		}
@@ -520,7 +520,7 @@ void pkgmirror::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUncompressFor
 			if (haveSize != newSize)
 			{
 				m_totalSize -= (newSize - haveSize);
-				SendFmt() << "Remaining download size: " << offttosH(m_totalSize) << "<br>\n";
+				SendFmt << "Remaining download size: " << offttosH(m_totalSize) << "<br>\n";
 			}
 		}
 	}
