@@ -103,7 +103,7 @@ void tDeleter::ModContents(mstring & contents, cmstring &cmd)
 				&& strtoul(tok.c_str()+2, &sep, 10)>0
 				&& sep && '=' == *sep)
 		{
-			files.push_back(UrlDecode(sep+1));
+			files.push_back(UrlUnescape(sep+1));
 		}
 	}
 
