@@ -23,6 +23,10 @@ using namespace MYSTD;
 //#warning FIXME, hack
 //#define NOCONCACHE
 
+#ifdef DEBUG
+volatile int nConCount(0), nDisconCount(0), nReuseCount(0);
+#endif
+
 tcpconnect::tcpconnect() :	m_conFd(-1), m_pConnStateObserver(NULL)
 {
 	m_proxy = acfg::proxy_info.sHost.empty() ? NULL : &acfg::proxy_info;
@@ -35,6 +39,40 @@ tcpconnect::~tcpconnect()
 }
 
 
+/*! \brief Helper to flush data stream contents reliable and close the connection then
+ * DUDES, who write TCP implementations... why can this just not be done easy and reliable? Why do we need hacks like termsocket?
+ For details, see: http://blog.netherlabs.nl/articles/2009/01/18/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable
+ *
+ */
+void termsocket(int fd)
+{
+	LOGSTART2s("::termsocket", fd);
+	if (fd < 0)
+		return;
+
+	fcntl(fd, F_SETFL, ~O_NONBLOCK & fcntl(fd, F_GETFL));
+	::shutdown(fd, SHUT_WR);
+	char buf[40];
+	LOG("waiting for peer to react");
+	while(true)
+	{
+		int r=recv(fd, buf, 40, MSG_WAITALL);
+		if(0 == r)
+			break;
+		if(r < 0)
+		{
+			if(errno == EINTR)
+				continue;
+			break; // XXX error case, actually
+		}
+	}
+
+	while (0 != ::close(fd))
+	{
+		if (errno != EINTR)
+			break;
+	};
+}
 
 static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t addrlen, time_t timeout)
 {
@@ -55,16 +93,20 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 
 	res = connect(sockfd, addr, addrlen);
 	if (res < 0) {
-		if (EINPROGRESS == errno) {
+		if (EINPROGRESS == errno)
+		{
 			for (;;) {
 				// Wait for connection.
 				FD_ZERO(&wfds);
 				FD_SET(sockfd, &wfds);
 				res = select(sockfd+1, NULL, &wfds, NULL, &tv);
-				if (res < 0) {
+				if (res < 0)
+				{
 					if (EINTR != errno)
 						return -1;
-				} else if (res > 0) {
+				}
+				else if (res > 0)
+				{
 					// Socket selected for writing.
 					int err;
 					socklen_t optlen = sizeof(err);
@@ -72,7 +114,8 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 					if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (void *)&err, &optlen) < 0)
 						return -1;
 
-					if (err) {
+					if (err)
+					{
 						errno = err;
 						return -1;
 					}
@@ -104,7 +147,7 @@ bool tcpconnect::Connect(const string & sHostname, const string & sPort, string
 			: CAddrInfo::CachedResolve(sHostname, sPort, sErrorMsg);
 	if(!dns)
 	{
-		USRDBG(4, sErrorMsg);
+		USRDBG(sErrorMsg);
 		return false; // sErrorMsg got the info already, no other chance to fix it
 	}
 
@@ -139,7 +182,9 @@ bool tcpconnect::Connect(const string & sHostname, const string & sPort, string
 
 			if (::connect_timeout(m_conFd, pInfo->ai_addr, pInfo->ai_addrlen, acfg::nettimeout) < 0)
 				continue;
-
+#ifdef DEBUG
+			__sync_fetch_and_add(&nConCount, 1);
+#endif
 			ldbg("connect() ok");
 			set_nb(m_conFd);
 			m_sHostName = sHostname;
@@ -163,43 +208,18 @@ void tcpconnect::Disconnect()
 {
 	LOGSTART("tcpconnect::_Disconnect");
 
+#ifdef DEBUG
+	if(m_conFd >=0)
+		__sync_fetch_and_add(&nDisconCount, 1);
+#endif
+
 	if(m_conFd>=0 && m_pConnStateObserver)
 		m_pConnStateObserver->JobRelease();
 
 	m_pConnStateObserver=NULL;
 	m_lastFile.reset();
 
-	forceShutdownClose(m_conFd);
-}
-
-void termsocket(int fd)
-{
-	LOGSTART2s("::termsocket", fd);
-	if (fd < 0)
-		return;
-
-	fcntl(fd, F_SETFL, ~O_NONBLOCK & fcntl(fd, F_GETFL));
-	::shutdown(fd, SHUT_WR);
-	char buf[40];
-	LOG("waiting for peer to react");
-	while(true)
-	{
-		int r=recv(fd, buf, 40, MSG_WAITALL);
-		if(0 == r)
-			break;
-		if(r < 0)
-		{
-			if(errno == EINTR)
-				continue;
-			break; // XXX error case, actually
-		}
-	}
-
-	while (0 != ::close(fd))
-	{
-		if (errno != EINTR)
-			break;
-	};
+	termsocket_quick(m_conFd);
 }
 
 lockable conPoolMx;
@@ -220,6 +240,8 @@ tConPool spareConPool;
 tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, cmstring &sPort,
 		mstring &sErrOut, bool *pbSecondHand, acfg::tRepoData::IHookHandler *pStateTracker)
 {
+	LOGSTART2s("tcpconnect::CreateConnected", "hostname: " << sHostname);
+
 	tTcpHandlePtr p;
 	bool bReused=false;
 	tHostHint key(sHostname, sPort);
@@ -243,6 +265,10 @@ tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, cmstring &sPort,
 			p=it->second.first;
 			spareConPool.erase(it);
 			bReused = true;
+			ldbg("got connection " << p.get() << " from the idle pool");
+#ifdef DEBUG
+			__sync_fetch_and_add(&nReuseCount, 1);
+#endif
 		}
 	}
 #endif
@@ -264,15 +290,35 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	if(!handle)
 		return;
 
+	LOGSTART2s("tcpconnect::RecycleIdleConnection", handle->m_sHostName);
+
 	if(handle->m_pConnStateObserver)
 	{
 		handle->m_pConnStateObserver->JobRelease();
 		handle->m_pConnStateObserver = NULL;
 	}
 
+	if(! acfg::persistoutgoing)
+	{
+		ldbg("not caching outgoing connections, drop " << handle.get());
+		handle.reset();
+		return;
+	}
+
+#ifdef DEBUG
+	if(check_read_state(handle->GetFD()))
+	{
+		acbuf checker;
+		checker.init(300000);
+		checker.sysread(handle->GetFD());
+
+	}
+#endif
+
 #ifndef NOCONCACHE
 	time_t now=GetTime();
 	lockguard __g(conPoolMx);
+	ldbg("caching connection " << handle.get());
 
 	// a DOS?
 	if(spareConPool.size()<50)
@@ -281,7 +327,7 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 				make_pair(handle, now)));
 
 #ifndef MINIBUILD
-		g_victor.RunLatestAt(now+TIME_SOCKET_EXPIRE_CLOSE);
+		g_victor.ScheduleFor(now+TIME_SOCKET_EXPIRE_CLOSE, cleaner::TYPE_EXCONNS);
 #endif
 
 	}
@@ -290,7 +336,7 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	handle.reset();
 }
 
-time_t tcpconnect::ExpireCache()
+time_t tcpconnect::BackgroundCleanup()
 {
 	lockguard __g(conPoolMx);
 	time_t now=GetTime();
@@ -339,3 +385,28 @@ void tcpconnect::KillLastFile()
 #endif
 }
 
+void tcpconnect::dump_status()
+{
+	lockguard __g(conPoolMx);
+	tSS msg;
+	msg << "TCP connection cache:\n";
+	for (tConPool::iterator it = spareConPool.begin(); it != spareConPool.end();++it)
+	{
+		if(! it->second.first)
+		{
+			msg << "[BAD HANDLE] recycle at " << it->second.second << "\n";
+			continue;
+		}
+
+		msg << it->second.first->m_conFd << ": for "
+				<< it->first.pHost << ":" << it->first.pPort
+				<< ", recycled at " << it->second.second << "\n";
+	}
+#ifdef DEBUG
+	msg << "dbg counts, con: " << __sync_fetch_and_add(&nConCount, 0)
+			<< " , discon: " << __sync_fetch_and_add(&nDisconCount, 0)
+			<< " , reuse: " << __sync_fetch_and_add(&nReuseCount, 0) << "\n";
+#endif
+
+	aclog::err(msg);
+}
